#!/usr/bin/env python3

import os, re, argparse, filecmp, json, glob
import subprocess as sp
import numpy as np
import dpgen.auto_test.lib.vasp as vasp
import dpgen.auto_test.lib.lammps as lammps
from pymatgen.core.structure import Structure
from pymatgen.analysis.elasticity.strain import Deformation, DeformedStructureSet, Strain

global_equi_name = '00.equi'
global_task_name = '07.SScurve'

def make_vasp(jdata, conf_dir, norm_def = 2e-3, shear_def = 5e-3) :
    fp_params = jdata['vasp_params']
    ecut = fp_params['ecut']
    ediff = fp_params['ediff']
    npar = fp_params['npar']
    kpar = fp_params['kpar']
    kspacing = fp_params['kspacing']
    kgamma = fp_params['kgamma']
    strain_start=jdata['strain_start']
    strain_end=jdata['strain_end']
    strain_step=jdata['strain_step']
    strain_direct=jdata['strain_direct']

    conf_path = os.path.abspath(conf_dir)
    conf_poscar = os.path.join(conf_path, 'POSCAR')
    # get equi poscar
    equi_path = re.sub('confs', global_equi_name, conf_path)
    equi_path = os.path.join(equi_path, 'vasp-k%.2f' % kspacing)
    equi_contcar = os.path.join(equi_path, 'CONTCAR')
    task_path = re.sub('confs', global_task_name, conf_path)
    task_path = os.path.join(task_path, 'vasp-k%.2f' % kspacing)
    os.makedirs(task_path, exist_ok=True)
    cwd = os.getcwd()
    os.chdir(task_path)
    if os.path.isfile('POSCAR') :
        os.remove('POSCAR')
    os.symlink(os.path.relpath(equi_contcar), 'POSCAR')
    os.chdir(cwd)
    task_poscar = os.path.join(task_path, 'POSCAR')
    # stress
    equi_outcar = os.path.join(equi_path, 'OUTCAR')
    stress = vasp.get_stress(equi_outcar)
    np.savetxt(os.path.join(task_path, 'equi.stress.out'), stress)
    # gen strcture
    ss = Structure.from_file(task_poscar)
    # gen defomations
    norm_strains=np.arange(strain_start,strain_end,strain_step)
    print('gen with norm '+str(norm_strains))
    deformations=[]
    for ii in norm_strains:
        strain = Strain.from_index_amount(strain_direct, ii)
        deformations.append(strain.get_deformation_matrix())
    deformed_structures = [defo.apply_to_structure(ss)
                                    for defo in deformations]
    n_dfm = len(deformed_structures)
    # gen incar
    fc = vasp.make_vasp_relax_incar(ecut, ediff, True, False, False, npar=npar, kpar=kpar, kspacing = None, kgamma = None)
    with open(os.path.join(task_path, 'INCAR'), 'w') as fp :
        fp.write(fc)
    # gen potcar
    with open(task_poscar,'r') as fp :
        lines = fp.read().split('\n')
        ele_list = lines[5].split()
    potcar_map = jdata['potcar_map']
    potcar_list = []
    for ii in ele_list :
        assert(os.path.exists(potcar_map[ii]))
        potcar_list.append(potcar_map[ii])
    with open(os.path.join(task_path,'POTCAR'), 'w') as outfile:
        for fname in potcar_list:
            with open(fname) as infile:
                outfile.write(infile.read())
    # gen kpoints
    fc = vasp.make_kspacing_kpoints(task_poscar, kspacing, kgamma)
    with open(os.path.join(task_path,'KPOINTS'), 'w') as fp:
        fp.write(fc)
    # gen tasks    
    cwd = os.getcwd()
    for ii in range(n_dfm) :
        # make dir
        dfm_path = os.path.join(task_path, 'dfm-%03d' % ii)
        os.makedirs(dfm_path, exist_ok=True)
        os.chdir(dfm_path)
        for jj in ['POSCAR', 'POTCAR', 'INCAR', 'KPOINTS'] :
            if os.path.isfile(jj):
                os.remove(jj)
        # make conf
        deformed_structures[ii].to('POSCAR', 'POSCAR')
        # record strain
        strain = Strain.from_deformation(deformations[ii])
        np.savetxt('strain.out', strain)
        # link incar, potcar, kpoints
        os.symlink(os.path.relpath(os.path.join(task_path, 'INCAR')), 'INCAR')
        os.symlink(os.path.relpath(os.path.join(task_path, 'POTCAR')), 'POTCAR')
        os.symlink(os.path.relpath(os.path.join(task_path, 'KPOINTS')), 'KPOINTS')
    cwd = os.getcwd()

def make_lammps(jdata, conf_dir,task_type) :
    fp_params = jdata['lammps_params']
    model_dir = fp_params['model_dir']
    type_map = fp_params['type_map'] 
    model_dir = os.path.abspath(model_dir)
    model_name =fp_params['model_name']
    deepmd_version = fp_params.get("deepmd_version", "0.12")
    if not model_name :
        models = glob.glob(os.path.join(model_dir, '*pb'))
        model_name = [os.path.basename(ii) for ii in models]
    else:
        models = [os.path.join(model_dir,ii) for ii in model_name]

    model_param = {'model_name' :      model_name,
                  'param_type':          fp_params['model_param_type'],
                  'deepmd_version' : deepmd_version}
    ntypes = len(type_map)
    strain_start=jdata['strain_start']
    strain_end=jdata['strain_end']
    strain_step=jdata['strain_step']
    strain_direct=jdata['strain_direct']

    conf_path = os.path.abspath(conf_dir)
    conf_poscar = os.path.join(conf_path, 'POSCAR')
    # get equi poscar
    equi_path = re.sub('confs', global_equi_name, conf_path)
    equi_path = os.path.join(equi_path, task_type)
    equi_dump = os.path.join(equi_path, 'dump.relax')
    task_path = re.sub('confs', global_task_name, conf_path)
    task_path = os.path.join(task_path, task_type)
    os.makedirs(task_path, exist_ok=True)
    task_poscar = os.path.join(task_path, 'POSCAR')
    lammps.poscar_from_last_dump(equi_dump, task_poscar, type_map)
    # get equi stress
    equi_log = os.path.join(equi_path, 'log.lammps')
    stress = lammps.get_stress(equi_log)
    np.savetxt(os.path.join(task_path, 'equi.stress.out'), stress)
    # gen strcture
    ss = Structure.from_file(task_poscar)
    # gen defomations
    norm_strains=np.arange(strain_start,strain_end,strain_step)
    print('gen with norm '+str(norm_strains))
    deformations=[]
    for ii in norm_strains:
        strain = Strain.from_index_amount(strain_direct, ii)
        deformations.append(strain.get_deformation_matrix())
    deformed_structures = [defo.apply_to_structure(ss)
                                    for defo in deformations]
    n_dfm = len(deformed_structures)
    # gen tasks    
    cwd = os.getcwd()
    # make lammps.in
    if task_type=='deepmd':
        fc = lammps.make_lammps_elastic('conf.lmp', 
                                    ntypes, 
                                    lammps.inter_deepmd,
                                    model_param)
    elif task_type =='meam':
        fc = lammps.make_lammps_elastic('conf.lmp', 
                                    ntypes, 
                                    lammps.inter_meam,
                                    model_param) 


    f_lammps_in = os.path.join(task_path, 'lammps.in')
    with open(f_lammps_in, 'w') as fp :
        fp.write(fc)
    cwd = os.getcwd()
    if task_type =='deepmd':
        os.chdir(task_path)
        for ii in model_name :
            if os.path.exists(ii) :
                os.remove(ii)
        for (ii,jj) in zip(models, model_name) :
                os.symlink(os.path.relpath(ii), jj)
        share_models = glob.glob(os.path.join(task_path, '*pb'))
    else:
        share_models = models

    for ii in range(n_dfm) :
        # make dir
        dfm_path = os.path.join(task_path, 'dfm-%03d' % ii)
        os.makedirs(dfm_path, exist_ok=True)
        os.chdir(dfm_path)
        for jj in ['conf.lmp', 'lammps.in'] + model_name :
            if os.path.isfile(jj):
                os.remove(jj)
        # make conf
        deformed_structures[ii].to('POSCAR', 'POSCAR')
        lammps.cvt_lammps_conf('POSCAR', 'conf.lmp')
        ptypes = vasp.get_poscar_types('POSCAR')
        lammps.apply_type_map('conf.lmp', type_map, ptypes)    
        # record strain
        strain = Strain.from_deformation(deformations[ii])
        np.savetxt('strain.out', strain)
        # link lammps.in
        os.symlink(os.path.relpath(f_lammps_in), 'lammps.in')
        # link models
        for (ii,jj) in zip(share_models, model_name) :
            os.symlink(os.path.relpath(ii), jj)
    cwd = os.getcwd()

def make_meam_lammps(jdata, conf_dir) :
    meam_potfile_dir = jdata['meam_potfile_dir']
    meam_potfile_dir = os.path.abspath(meam_potfile_dir)
    meam_potfile = jdata['meam_potfile']
    meam_potfile = [os.path.join(meam_potfile_dir,ii) for ii in meam_potfile]
    meam_potfile_name = jdata['meam_potfile']
    type_map = jdata['meam_type_map']
    ntypes = len(type_map)
    meam_param = {'meam_potfile' :      jdata['meam_potfile'],
                  'meam_type':          jdata['meam_param_type']}

    norm_def = jdata['norm_deform']
    shear_def = jdata['shear_deform']

    conf_path = os.path.abspath(conf_dir)
    conf_poscar = os.path.join(conf_path, 'POSCAR')
    # get equi poscar
    equi_path = re.sub('confs', global_equi_name, conf_path)
    equi_path = os.path.join(equi_path, 'meam')
    equi_dump = os.path.join(equi_path, 'dump.relax')
    task_path = re.sub('confs', global_task_name, conf_path)
    task_path = os.path.join(task_path, 'meam')
    os.makedirs(task_path, exist_ok=True)
    task_poscar = os.path.join(task_path, 'POSCAR')
    lammps.poscar_from_last_dump(equi_dump, task_poscar, type_map)
    # get equi stress
    equi_log = os.path.join(equi_path, 'log.lammps')
    stress = lammps.get_stress(equi_log)
    np.savetxt(os.path.join(task_path, 'equi.stress.out'), stress)
    # gen strcture
    # ss = Structure.from_file(conf_poscar)
    # print(ss)
    # ss = ss.from_file(task_poscar)
    # print(ss)
    ss = Structure.from_file(task_poscar)
    # gen defomations
    norm_strains = [-norm_def, -0.5*norm_def, 0.5*norm_def, norm_def]
    shear_strains = [-shear_def, -0.5*shear_def, 0.5*shear_def, shear_def]
    print('gen with norm '+str(norm_strains))
    print('gen with shear '+str(shear_strains))
    dfm_ss = DeformedStructureSet(ss, 
                                  symmetry = False, 
                                  norm_strains = norm_strains,
                                  shear_strains = shear_strains)
    n_dfm = len(dfm_ss)
    # gen tasks    
    cwd = os.getcwd()
    # make lammps.in
    fc = lammps.make_lammps_elastic('conf.lmp', 
                                    ntypes, 
                                    lammps.inter_meam,
                                    meam_param)        
    f_lammps_in = os.path.join(task_path, 'lammps.in')
    with open(f_lammps_in, 'w') as fp :
        fp.write(fc)
    cwd = os.getcwd()
    for ii in range(n_dfm) :
        # make dir
        dfm_path = os.path.join(task_path, 'dfm-%03d' % ii)
        os.makedirs(dfm_path, exist_ok=True)
        os.chdir(dfm_path)
        for jj in ['conf.lmp', 'lammps.in'] + meam_potfile_name :
            if os.path.isfile(jj):
                os.remove(jj)
        # make conf
        dfm_ss.deformed_structures[ii].to('POSCAR', 'POSCAR')
        lammps.cvt_lammps_conf('POSCAR', 'conf.lmp')
        ptypes = vasp.get_poscar_types('POSCAR')
        lammps.apply_type_map('conf.lmp', type_map, ptypes)    
        # record strain
        strain = Strain.from_deformation(dfm_ss.deformations[ii])
        np.savetxt('strain.out', strain)
        # link lammps.in
        os.symlink(os.path.relpath(f_lammps_in), 'lammps.in')
        # link models
        for (ii,jj) in zip(meam_potfile, meam_potfile_name) :
            os.symlink(os.path.relpath(ii), jj)
    cwd = os.getcwd()

    
def _main() :
    parser = argparse.ArgumentParser(
        description="gen 07.SScurve")
    parser.add_argument('TASK', type=str,
                        help='the task of generation, vasp or lammps')
    parser.add_argument('PARAM', type=str,
                        help='json parameter file')
    parser.add_argument('CONF', type=str,
                        help='the path to conf')
    args = parser.parse_args()

    with open (args.PARAM, 'r') as fp :
        jdata = json.load (fp)

    print('generate %s task with conf %s' % (args.TASK, args.CONF))
    if args.TASK == 'vasp':
        make_vasp(jdata, args.CONF)               
    elif args.TASK == 'deepmd' or args.TASK == 'meam' :
        make_lammps(jdata, args.CONF,args.TASK)
    #elif args.TASK == 'meam' :
    #    make_meam_lammps(jdata, args.CONF)
    else :
        raise RuntimeError("unknow task ", args.TASK)
    
if __name__ == '__main__' :
    _main()




#!/usr/bin/env python3

import random, os, sys
import dpdata
import subprocess as sp
import dpgen.auto_test.lib.util as util
from distutils.version import LooseVersion

def cvt_lammps_conf(fin, fout, ofmt = 'lammps/data'):
    """
    Format convert from fin to fout, specify the output format by ofmt
    Imcomplete situation
    """
    supp_ofmt = ['lammps/dump', 'lammps/data', 'vasp/poscar']
    supp_exts = ['dump', 'lmp', 'poscar/POSCAR']

    if 'dump' in fout :
        ofmt = 'lammps/dump'
    elif 'lmp' in fout :
        ofmt = 'lammps/data'
    elif 'poscar' in fout or 'POSCAR' in fout :
        ofmt = 'vasp/poscar'
    if not ofmt in supp_ofmt :
        raise RuntimeError ("output format " + ofmt + " is not supported. use one of " + str(supp_ofmt))

    if 'lmp' in fout :
        d_poscar=dpdata.System(fin, fmt = 'vasp/poscar')
        d_poscar.to_lammps_lmp(fout, frame_idx=0)
    elif 'poscar' in fout or 'POSCAR' in fout:
        d_dump=dpdata.System(fin, fmt = 'lammps/dump')
        d_dump.to_vasp_poscar(fout, frame_idx=-1)

def apply_type_map(conf_file, deepmd_type_map, ptypes) :
    """
    apply type map.
    conf_file:          conf file converted from POSCAR
    deepmd_type_map:    deepmd atom type map
    ptypes:             atom types defined in POSCAR
    """
    natoms = _get_conf_natom(conf_file)
    ntypes = len(deepmd_type_map)
    with open(conf_file, 'r') as fp:
        lines = fp.read().split('\n')
    # with open(conf_file+'.bk', 'w') as fp:
    #     fp.write("\n".join(lines))
    new_lines = lines
    # revise ntypes
    idx_ntypes = -1
    for idx, ii in enumerate(lines) :
        if 'atom types' in ii :
            idx_ntypes = idx
    if idx_ntypes == -1 :
        raise RuntimeError("cannot find the entry 'atom types' in ", conf_file)
    words = lines[idx_ntypes].split()
    words[0] = str(ntypes)
    new_lines[idx_ntypes] = " ".join(words)
    # find number of atoms
    idx_atom_entry = -1
    for idx, ii in enumerate(lines) :
        if 'Atoms' in ii :
            idx_atom_entry = idx
    if idx_atom_entry == -1 :
        raise RuntimeError("cannot find the entry 'Atoms' in ", conf_file)
    # revise atom type
    for idx in range(idx_atom_entry+2, idx_atom_entry+2+natoms) :
        ii = lines[idx]
        words = ii.split()
        assert(len(words) >= 5)
        old_id = int(words[1])
        new_id = deepmd_type_map.index(ptypes[old_id-1])+1
        words[1] = str(new_id)
        ii = " ".join(words)
        new_lines[idx] = ii
    with open(conf_file, 'w') as fp:
        fp.write("\n".join(new_lines))

def _get_ntype(conf) :
    with open(conf, 'r') as fp:
        lines = fp.read().split('\n')
    for ii in lines :
        if "atom types" in ii :
            return int(ii.split()[0])
    raise RuntimeError("cannot find line indicate atom types in ", conf)

def _get_conf_natom(conf) :
    with open(conf, 'r') as fp:
        lines = fp.read().split('\n')
    for ii in lines :
        if "atoms" in ii :
            return int(ii.split()[0])
    raise RuntimeError("cannot find line indicate atom types in ", conf)

def inter_deepmd(param) :
    models = param["model_name"]
    deepmd_version = param["deepmd_version"]
    ret = "pair_style deepmd "
    model_list = ""
    for ii in models:
        model_list += ii + " "
    if LooseVersion(deepmd_version) < LooseVersion('1'):
        ## DeePMD-kit version == 0.x
        if len(models) > 1 :
            ret += '%s 10 model_devi.out\n' % model_list
        else :
            ret += models[0] + '\n'
    else:
        ## DeePMD-kit version >= 1
        if len(models) > 1:
            ret += "%s out_freq 10 out_file model_devi.out\n" % model_list
        else:
            ret += models[0] + '\n'
    ret += "pair_coeff\n"
    return ret

def inter_meam(param) :
    ret = ""
    line = "pair_style      meam \n"
    line+= "pair_coeff      * * %s " % param['model_name'][0]
    for ii in param['param_type'] :
        line += ii + ' '
    line+= "%s " % param['model_name'][1]
    for ii in param['param_type'] :
        line += ii + ' '
    line+= '\n'
    ret += line
    return ret

def make_lammps_eval(conf, ntypes, interaction, param) :
    """
    make lammps input for equilibritation
    """
    ret = ""
    ret += "clear\n"
    ret += "units 	metal\n"
    ret += "dimension	3\n"
    ret += "boundary	p	p    p\n"
    ret += "atom_style	atomic\n"
    ret += "box         tilt large\n"
    ret += "read_data   %s\n" % conf
    for ii in range(ntypes) :
        ret += "mass            %d 1\n" % (ii+1)
    ret += "neigh_modify    every 1 delay 0 check no\n"
    ret += interaction(param)
    ret += "compute         mype all pe\n"
    ret += "thermo          100\n"
    ret += "thermo_style    custom step pe pxx pyy pzz pxy pxz pyz lx ly lz vol c_mype\n"
    ret += "run    0\n"
    ret += "variable        N equal count(all)\n"
    ret += "variable        V equal vol\n"
    ret += "variable        E equal \"c_mype\"\n"
    ret += "variable        tmplx equal lx\n"
    ret += "variable        tmply equal ly\n"
    ret += "variable        Pxx equal pxx\n"
    ret += "variable        Pyy equal pyy\n"
    ret += "variable        Pzz equal pzz\n"
    ret += "variable        Pxy equal pxy\n"
    ret += "variable        Pxz equal pxz\n"
    ret += "variable        Pyz equal pyz\n"
    ret += "variable        Epa equal ${E}/${N}\n"
    ret += "variable        Vpa equal ${V}/${N}\n"
    ret += "variable        AA equal (${tmplx}*${tmply})\n"
    ret += "print \"All done\"\n"
    ret += "print \"Total number of atoms = ${N}\"\n"
    ret += "print \"Final energy per atoms = ${Epa}\"\n"
    ret += "print \"Final volume per atoms = ${Vpa}\"\n"
    ret += "print \"Final Base area = ${AA}\"\n"
    ret += "print \"Final Stress (xx yy zz xy xz yz) = ${Pxx} ${Pyy} ${Pzz} ${Pxy} ${Pxz} ${Pyz}\"\n"
    return ret


def make_lammps_equi(conf, ntypes, interaction, param,
                     etol=1e-12, ftol=1e-6,
                     maxiter=5000, maxeval=500000,
                     change_box = True) :
    """
    make lammps input for equilibritation
    """
    ret = ""
    ret += "clear\n"
    ret += "units 	metal\n"
    ret += "dimension	3\n"
    ret += "boundary	p	p    p\n"
    ret += "atom_style	atomic\n"
    ret += "box         tilt large\n"
    ret += "read_data   %s\n" % conf
    for ii in range(ntypes) :
        ret += "mass            %d 1\n" % (ii+1)
    ret += "neigh_modify    every 1 delay 0 check no\n"
    ret += interaction(param)
    ret += "compute         mype all pe\n"
    ret += "thermo          100\n"
    ret += "thermo_style    custom step pe pxx pyy pzz pxy pxz pyz lx ly lz vol c_mype\n"
    ret += "dump            1 all custom 100 dump.relax id type xs ys zs fx fy fz\n"
    ret += "min_style       cg\n"
    if change_box :
        ret += "fix             1 all box/relax iso 0.0 \n"
        ret += "minimize        %e %e %d %d\n" % (etol, ftol, maxiter, maxeval)
        ret += "fix             1 all box/relax aniso 0.0 \n"
    ret += "minimize        %e %e %d %d\n" % (etol, ftol, maxiter, maxeval)
    ret += "variable        N equal count(all)\n"
    ret += "variable        V equal vol\n"
    ret += "variable        E equal \"c_mype\"\n"
    ret += "variable        tmplx equal lx\n"
    ret += "variable        tmply equal ly\n"
    ret += "variable        Pxx equal pxx\n"
    ret += "variable        Pyy equal pyy\n"
    ret += "variable        Pzz equal pzz\n"
    ret += "variable        Pxy equal pxy\n"
    ret += "variable        Pxz equal pxz\n"
    ret += "variable        Pyz equal pyz\n"
    ret += "variable        Epa equal ${E}/${N}\n"
    ret += "variable        Vpa equal ${V}/${N}\n"
    ret += "variable        AA equal (${tmplx}*${tmply})\n"
    ret += "print \"All done\"\n"
    ret += "print \"Total number of atoms = ${N}\"\n"
    ret += "print \"Final energy per atoms = ${Epa}\"\n"
    ret += "print \"Final volume per atoms = ${Vpa}\"\n"
    ret += "print \"Final Base area = ${AA}\"\n"
    ret += "print \"Final Stress (xx yy zz xy xz yz) = ${Pxx} ${Pyy} ${Pzz} ${Pxy} ${Pxz} ${Pyz}\"\n"
    return ret

def make_lammps_elastic(conf, ntypes, interaction, param,
                        etol=1e-12, ftol=1e-6,
                        maxiter=5000, maxeval=500000) :
    """
    make lammps input for elastic calculation
    """
    ret = ""
    ret += "clear\n"
    ret += "units 	metal\n"
    ret += "dimension	3\n"
    ret += "boundary	p	p    p\n"
    ret += "atom_style	atomic\n"
    ret += "box         tilt large\n"
    ret += "read_data   %s\n" % conf
    for ii in range(ntypes) :
        ret += "mass            %d 1\n" % (ii+1)
    ret += "neigh_modify    every 1 delay 0 check no\n"
    ret += interaction(param)
    ret += "compute         mype all pe\n"
    ret += "thermo          100\n"
    ret += "thermo_style    custom step pe pxx pyy pzz pxy pxz pyz lx ly lz vol c_mype\n"
    ret += "dump            1 all custom 100 dump.relax id type xs ys zs fx fy fz\n"
    ret += "min_style       cg\n"
    ret += "minimize        %e %e %d %d\n" % (etol, ftol, maxiter, maxeval)
    ret += "variable        N equal count(all)\n"
    ret += "variable        V equal vol\n"
    ret += "variable        E equal \"c_mype\"\n"
    ret += "variable        Pxx equal pxx\n"
    ret += "variable        Pyy equal pyy\n"
    ret += "variable        Pzz equal pzz\n"
    ret += "variable        Pxy equal pxy\n"
    ret += "variable        Pxz equal pxz\n"
    ret += "variable        Pyz equal pyz\n"
    ret += "variable        Epa equal ${E}/${N}\n"
    ret += "variable        Vpa equal ${V}/${N}\n"
    ret += "print \"All done\"\n"
    ret += "print \"Total number of atoms = ${N}\"\n"
    ret += "print \"Final energy per atoms = ${Epa}\"\n"
    ret += "print \"Final volume per atoms = ${Vpa}\"\n"
    ret += "print \"Final Stress (xx yy zz xy xz yz) = ${Pxx} ${Pyy} ${Pzz} ${Pxy} ${Pxz} ${Pyz}\"\n"
    return ret

def make_lammps_press_relax(conf, ntypes, scale2equi, interaction, param,
                            B0 = 70, bp = 0,
                            etol=1e-12, ftol=1e-6,
                            maxiter=5000, maxeval=500000) :
    """
    make lammps input for relaxation at a certain volume
    scale2equi: the volume scale with respect to equilibrium volume
    """
    ret = ""
    ret += "clear\n"
    ret += "variable        GPa2bar	equal 1e4\n"
    ret += "variable        B0		equal %f\n" % B0
    ret += "variable        bp		equal %f\n" % bp
    ret += "variable	    xx		equal %f\n" % scale2equi
    ret += "variable        yeta	equal 1.5*(${bp}-1)\n"
    ret += "variable        Px0		equal 3*${B0}*(1-${xx})/${xx}^2*exp(${yeta}*(1-${xx}))\n"
    ret += "variable        Px		equal ${Px0}*${GPa2bar}\n"
    ret += "units       metal\n"
    ret += "dimension   3\n"
    ret += "boundary	p p p\n"
    ret += "atom_style	atomic\n"
    ret += "box         tilt large\n"
    ret += "read_data   %s\n" % conf
    for ii in range(ntypes) :
        ret += "mass            %d 1\n" % (ii+1)
    ret += "neigh_modify    every 1 delay 0 check no\n"
    ret += interaction(param)
    ret += "compute         mype all pe\n"
    ret += "thermo          100\n"
    ret += "thermo_style    custom step pe pxx pyy pzz pxy pxz pyz lx ly lz vol c_mype\n"
    ret += "dump            1 all custom 100 dump.relax id type xs ys zs fx fy fz\n"
    ret += "min_style       cg\n"
    ret += "fix             1 all box/relax iso ${Px} \n"
    ret += "minimize        %e %e %d %d\n" % (etol, ftol, maxiter, maxeval)
    ret += "fix             1 all box/relax aniso ${Px} \n"
    ret += "minimize        %e %e %d %d\n" % (etol, ftol, maxiter, maxeval)
    ret += "variable        N equal count(all)\n"
    ret += "variable        V equal vol\n"
    ret += "variable        E equal \"c_mype\"\n"
    ret += "variable        Pxx equal pxx\n"
    ret += "variable        Pyy equal pyy\n"
    ret += "variable        Pzz equal pzz\n"
    ret += "variable        Pxy equal pxy\n"
    ret += "variable        Pxz equal pxz\n"
    ret += "variable        Pyz equal pyz\n"
    ret += "variable        Epa equal ${E}/${N}\n"
    ret += "variable        Vpa equal ${V}/${N}\n"
    ret += "print \"All done\"\n"
    ret += "print \"Total number of atoms  = ${N}\"\n"
    ret += "print \"Relax at Press         = ${Px} Bar\"\n"
    ret += "print \"Final energy per atoms = ${Epa} eV\"\n"
    ret += "print \"Final volume per atoms = ${Vpa} A^3\"\n"
    ret += "print \"Final Stress (xx yy zz xy xz yz) = ${Pxx} ${Pyy} ${Pzz} ${Pxy} ${Pxz} ${Pyz}\"\n"
    return ret

def make_lammps_phonon(conf, masses, interaction, param,
                        etol=1e-12, ftol=1e-6,
                        maxiter=5000, maxeval=500000):
    """
    make lammps input for elastic calculation
    """
    ret = ""
    ret += "clear\n"
    ret += "units 	metal\n"
    ret += "dimension	3\n"
    ret += "boundary	p	p    p\n"
    ret += "atom_style	atomic\n"
    ret += "box         tilt large\n"
    ret += "read_data   %s\n" % conf
    ntypes=len(masses)
    for ii in range(ntypes) :
        ret += "mass            %d %f\n" % (ii+1,masses[ii])
    ret += "neigh_modify    every 1 delay 0 check no\n"
    ret += interaction(param)
    return ret

def _get_epa (lines) :
    for ii in lines:
        if ("Final energy per atoms" in ii) and (not 'print' in ii):
            return float(ii.split('=')[1].split()[0])
    raise RuntimeError("cannot find key \"Final energy per atoms\" in lines, something wrong")

def _get_vpa (lines) :
    for ii in lines :
        if ("Final volume per atoms" in ii) and (not 'print' in ii):
            return float(ii.split('=')[1].split()[0])
    raise RuntimeError("cannot find key \"Final volume per atoms\" in lines, something wrong")

def _get_natoms (lines) :
    for ii in lines:
        if ("Total number of atoms" in ii) and (not 'print' in ii):
            return int(ii.split('=')[1].split()[0])
    raise RuntimeError("cannot find key \"Total number of atoms\" in lines, something wrong")

def get_nev (log) :
    """
    get natoms, energy_per_atom and volume_per_atom from lammps log
    """
    with open(log, 'r') as fp:
        lines = fp.read().split('\n')
    epa = _get_epa(lines)
    vpa = _get_vpa(lines)
    natoms = _get_natoms(lines)
    return natoms, epa, vpa

def get_base_area (log) :
    """
    get base area
    """
    with open(log, 'r') as fp:
        lines = fp.read().split('\n')
    for ii in lines:
        if ("Final Base area" in ii) and (not 'print' in ii):
            return float(ii.split('=')[1].split()[0])

def get_stress(log) :
    """
    get stress from lammps log
    """
    with open(log, 'r') as fp :
        lines = fp.read().split('\n')
    for ii in lines :
        if ('Final Stress' in ii) and (not 'print' in ii):
            vstress = [float(jj) for jj in ii.split('=')[1].split()]
    stress = util.voigt_to_stress(vstress)
    return stress

def poscar_from_last_dump(dump, poscar_out, deepmd_type_map) :
    """
    get poscar from the last frame of a lammps MD traj (dump format)
    """
    with open(dump, 'r') as fp :
        lines = fp.read().split('\n')
    step_idx = -1
    for idx,ii in enumerate(lines) :
        if 'ITEM: TIMESTEP' in ii :
            step_idx = idx
    if step_idx == -1 :
        raise RuntimeError("cannot find timestep in lammps dump, something wrong")
    with open('tmp_dump', 'w') as fp:
        fp.write("\n".join(lines[step_idx:]))
    cvt_lammps_conf('tmp_dump', poscar_out, ofmt='vasp')
    os.remove('tmp_dump')
    with open(poscar_out, 'r') as fp:
        lines = fp.read().split('\n')
    types = [ deepmd_type_map[int(ii.split('_')[1])] for ii in lines[5].split()]
    lines[5] = " ".join(types)
    with open(poscar_out, 'w') as fp:
        lines = fp.write("\n".join(lines))


def check_finished_new(fname,keyword):
    with open(fname, 'r') as fp :
        lines = fp.read().split('\n')
    flag=False
    for jj in lines:
        if (keyword in jj) and (not 'print' in jj):
            flag=True
    return flag

def check_finished(fname):
    with open(fname, 'r') as fp:
        return 'Total wall time:' in fp.read()



#!/usr/bin/python3
import os
import warnings
import numpy as np
import dpgen.auto_test.lib.lammps as lammps
import dpgen.auto_test.lib.util as util
from dpgen.generator.lib.vasp import incar_upper
from pymatgen.io.vasp import Incar,Kpoints,Potcar

class OutcarItemError(Exception):
    pass

# def get_poscar(conf_dir) :
#     conf_path = os.path.abspath(conf_dir)
#     poscar_out = os.path.join(conf_path, 'POSCAR')
#     contcar = os.path.join(conf_path, 'POSCAR')
#     lmp_dump = glob.glob(os.path.join(conf_path, 'dump.*'))
#     lmp_dump.sort()
#     if os.path.isfile(poscar_out) :
#         pass
#     elif os.path.isfile(contcar) :
#         os.symlink(contcar, poscar_out)
#     elif len(lmp_dump) == 1 :
#         dump_file = lmp_dump[0]
#         lammps.poscar_from_last_dump(dump_file, task_poscar, deepmd_type_map)

def regulate_poscar(poscar_in, poscar_out) :
    with open(poscar_in, 'r') as fp:
        lines = fp.read().split('\n')
    names = lines[5].split()
    counts = [int(ii) for ii in lines[6].split()]
    assert(len(names) == len(counts))
    uniq_name = []
    for ii in names :
        if not (ii in uniq_name) :
            uniq_name.append(ii)
    uniq_count = np.zeros(len(uniq_name), dtype = int)
    for nn,cc in zip(names,counts) :
        uniq_count[uniq_name.index(nn)] += cc
    natoms = np.sum(uniq_count)
    posis = lines[8:8+natoms]
    all_lines = []
    for ele in uniq_name:
        ele_lines = []
        for ii in posis :
            ele_name = ii.split()[-1]
            if ele_name == ele :
                ele_lines.append(ii)
        all_lines += ele_lines
    all_lines.append('')
    ret = lines[0:5]
    ret.append(" ".join(uniq_name))
    ret.append(" ".join([str(ii) for ii in uniq_count]))
    ret.append("Direct")
    ret += all_lines
    with open(poscar_out, 'w') as fp:
        fp.write("\n".join(ret))

def sort_poscar(poscar_in, poscar_out, new_names) :
    with open(poscar_in, 'r') as fp:
        lines = fp.read().split('\n')
    names = lines[5].split()
    counts = [int(ii) for ii in lines[6].split()]
    new_counts = np.zeros(len(counts), dtype = int)
    for nn,cc in zip(names,counts) :
        new_counts[new_names.index(nn)] += cc
    natoms = np.sum(new_counts)
    posis = lines[8:8+natoms]
    all_lines = []
    for ele in new_names:
        ele_lines = []
        for ii in posis :
            ele_name = ii.split()[-1]
            if ele_name == ele :
                ele_lines.append(ii)
        all_lines += ele_lines
    all_lines.append('')
    ret = lines[0:5]
    ret.append(" ".join(new_names))
    ret.append(" ".join([str(ii) for ii in new_counts]))
    ret.append("Direct")
    ret += all_lines
    with open(poscar_out, 'w') as fp:
        fp.write("\n".join(ret))

def perturb_xz (poscar_in, poscar_out, pert = 0.01) :
    with open(poscar_in, 'r') as fp:
        lines = fp.read().split('\n')
    zz = lines[4]
    az = [float(ii) for ii in zz.split()]
    az[0] += pert
    zz = [str(ii) for ii in az]
    zz = " ".join(zz)
    lines[4] = zz
    with open(poscar_out, 'w') as fp:
        fp.write("\n".join(lines))

def reciprocal_box(box) :
    rbox = np.linalg.inv(box)
    rbox = rbox.T
    # rbox = rbox / np.linalg.det(box)
    # print(np.matmul(box, rbox.T))
    # print(rbox)
    return rbox

def make_kspacing_kpoints(poscar, kspacing, kgamma) :
    if type(kspacing) is not list:
        kspacing = [kspacing, kspacing, kspacing]
    with open(poscar, 'r') as fp:
        lines = fp.read().split('\n')
    scale = float(lines[1])
    box = []
    for ii in range(2,5) :
        box.append([float(jj) for jj in lines[ii].split()[0:3]])
    box = np.array(box)
    box *= scale
    rbox = reciprocal_box(box)
    kpoints = [max(1,(np.ceil(2 * np.pi * np.linalg.norm(ii) / ks).astype(int))) for ii,ks in zip(rbox,kspacing)]
    ret = make_vasp_kpoints(kpoints, kgamma)
    return ret

def get_energies (fname) :
    if not check_finished(fname):
        warnings.warn("incomplete outcar: "+fname)
    with open(fname, 'r') as fp:
        lines = fp.read().split('\n')
    try :
        ener = _get_energies(lines)
        return ener
    except OutcarItemError :
        return None

def get_boxes (fname) :
    if not check_finished(fname):
        warnings.warn("incomplete outcar: "+fname)
    with open(fname, 'r') as fp:
        lines = fp.read().split('\n')
    try :
        ener = _get_boxes(lines)
        return ener
    except OutcarItemError :
        return None

def get_nev(fname) :
    if not check_finished(fname):
        warnings.warn("incomplete outcar: "+fname)
    with open(fname, 'r') as fp:
        lines = fp.read().split('\n')
    try:
        natoms = _get_natoms(lines)
        vol = _get_volumes(lines)[-1]
        ener = _get_energies(lines)[-1]
        return natoms, ener/natoms, vol/natoms
    except OutcarItemError:
        raise OutcarItemError("cannot find the result, please check the OUTCAR")
    # print(fname, natoms, vol, ener)

def get_stress(fname) :
    if not check_finished(fname):
        warnings.warn("incomplete outcar: "+fname)
    with open(fname, 'r') as fp:
        lines = fp.read().split('\n')
    try:
        stress = _get_stress(lines)[-1]
        return stress
    except OutcarItemError:
        return None

def check_finished(fname) :
    with open(fname, 'r') as fp:
        return 'Elapsed time (sec):' in fp.read()

def _get_natoms(lines) :
    ipt = None
    for ii in lines:
        if 'ions per type' in ii :
            ipt = [int(jj) for jj in ii.split()[4:]]
            return sum(ipt)
    raise OutcarItemError("cannot find item 'ions per type'")

def _get_energies(lines) :
    items = []
    for ii in lines:
        if 'free  energy   TOTEN' in ii:
            items.append(float (ii.split()[4]))
    if len(items) == 0:
        raise OutcarItemError("cannot find item 'free  energy   TOTEN'")
    return items

def _split_box_line(line) :
    return [float(line[0:16]), float(line[16:29]), float(line[29:42])]

def _get_boxes(lines) :
    items = []
    for idx,ii in enumerate(lines):
        tmp_box = []
        if 'direct lattice vectors' in ii :
            tmp_box.append(_split_box_line(lines[idx+1]))
            tmp_box.append(_split_box_line(lines[idx+2]))
            tmp_box.append(_split_box_line(lines[idx+3]))
            items.append(tmp_box)
    return np.array(items)

def _get_volumes(lines) :
    items = []
    for ii in lines:
        if 'volume of cell' in ii:
            items.append(float (ii.split()[4]))
    if len(items) == 0:
        raise OutcarItemError("cannot find item 'volume of cell'")
    return items

def _get_stress(lines) :
    items = []
    for ii in lines:
        if 'in kB' in ii:
            sv = [float(jj) for jj in ii.split()[2:8]]
            tmp = sv[4]
            sv[4] = sv[5]
            sv[5] = tmp
            items.append(util.voigt_to_stress(sv))
    if len(items) == 0:
        raise OutcarItemError("cannot find item 'in kB'")
    return items

def _compute_isif (relax_ions,
                  relax_shape,
                  relax_volume) :
    if   (relax_ions) and (not relax_shape) and (not relax_volume) :
        isif = 2
    elif (relax_ions) and (relax_shape) and (relax_volume) :
        isif = 3
    elif (relax_ions) and (relax_shape) and (not relax_volume) :
        isif = 4
    elif (not relax_ions) and (relax_shape) and (not relax_volume) :
        isif = 5
    elif (not relax_ions) and (relax_shape) and (relax_volume) :
        isif = 6
    elif (not relax_ions) and (not relax_shape) and (relax_volume) :
        isif = 7
    else :
        raise ValueError("unknow relax style")
    return isif

def make_vasp_static_incar (ecut, ediff,
                            npar, kpar,
                            kspacing = 0.5, kgamma = True,
                            ismear = 1, sigma = 0.2) :
    isif = 2
    ret = ''
    ret += 'PREC=A\n'
    ret += 'ENCUT=%d\n' % ecut
    ret += '# ISYM=0\n'
    ret += 'ALGO=normal\n'
    ret += 'EDIFF=%e\n' % ediff
    ret += 'EDIFFG=-0.01\n'
    ret += 'LREAL=A\n'
    ret += 'NPAR=%d\n' % npar
    ret += 'KPAR=%d\n' % kpar
    ret += "\n"
    ret += 'ISMEAR=%d\n' % ismear
    ret += 'SIGMA=%f\n' % sigma
    ret += "\n"
    ret += 'ISTART=0\n'
    ret += 'ICHARG=2\n'
    ret += 'NELMIN=6\n'
    ret += 'ISIF=%d\n' % isif
    ret += 'IBRION=-1\n'
    ret += "\n"
    ret += 'NSW=0\n'
    ret += "\n"
    ret += 'LWAVE=F\n'
    ret += 'LCHARG=F\n'
    ret += 'PSTRESS=0\n'
    ret += "\n"
    if kspacing is not None :
        ret += 'KSPACING=%f\n' % kspacing
    if kgamma is not None :
        if kgamma:
            ret += 'KGAMMA=T\n'
        else :
            ret += 'KGAMMA=F\n'
    return ret

def make_vasp_relax_incar (ecut, ediff,
                           relax_ion, relax_shape, relax_volume,
                           npar, kpar,
                           kspacing = 0.5, kgamma = True,
                           ismear = 1, sigma = 0.22) :
    isif = _compute_isif(relax_ion, relax_shape, relax_volume)
    ret = ''
    ret += 'PREC=A\n'
    ret += 'ENCUT=%d\n' % ecut
    ret += '# ISYM=0\n'
    ret += 'ALGO=normal\n'
    ret += 'EDIFF=%e\n' % ediff
    ret += 'EDIFFG=-0.01\n'
    ret += 'LREAL=A\n'
    ret += 'NPAR=%d\n' % npar
    ret += 'KPAR=%d\n' % kpar
    ret += "\n"
    ret += 'ISMEAR=%d\n' % ismear
    ret += 'SIGMA=%f\n' % sigma
    ret += "\n"
    ret += 'ISTART=0\n'
    ret += 'ICHARG=2\n'
    ret += 'NELM=100\n'
    ret += 'NELMIN=6\n'
    ret += 'ISIF=%d\n' % isif
    ret += 'IBRION=2\n'
    ret += "\n"
    ret += 'NSW=50\n'
    ret += "\n"
    ret += 'LWAVE=F\n'
    ret += 'LCHARG=F\n'
    ret += 'PSTRESS=0\n'
    ret += "\n"
    if kspacing is not None :
        ret += 'KSPACING=%f\n' % kspacing
    if kgamma is not None :
        if kgamma:
            ret += 'KGAMMA=T\n'
        else :
            ret += 'KGAMMA=F\n'
    return ret

def make_vasp_phonon_incar (ecut, ediff,
                            npar, kpar,
                            kspacing = 0.5, kgamma = True,
                            ismear = 1, sigma = 0.2) :
    isif = 2
    ret = ''
    ret += 'PREC=A\n'
    ret += 'ENCUT=%d\n' % ecut
    ret += '# ISYM=0\n'
    ret += 'ALGO=normal\n'
    ret += 'EDIFF=%e\n' % ediff
    ret += 'EDIFFG=-0.01\n'
    ret += 'LREAL=A\n'
    #ret += 'NPAR=%d\n' % npar
    ret += 'KPAR=%d\n' % kpar
    ret += "\n"
    ret += 'ISMEAR=%d\n' % ismear
    ret += 'SIGMA=%f\n' % sigma
    ret += "\n"
    ret += 'ISTART=0\n'
    ret += 'ICHARG=2\n'
    ret += 'NELMIN=4\n'
    ret += 'ISIF=%d\n' % isif
    ret += 'IBRION=8\n'
    ret += "\n"
    ret += 'NSW=1\n'
    ret += "\n"
    ret += 'LWAVE=F\n'
    ret += 'LCHARG=F\n'
    ret += 'PSTRESS=0\n'
    ret += "\n"
    if kspacing is not None :
        ret += 'KSPACING=%f\n' % kspacing
    if kgamma is not None :
        if kgamma:
            ret += 'KGAMMA=T\n'
        else :
            ret += 'KGAMMA=F\n'
    return ret

def get_poscar_types (fname) :
    with open(fname, 'r') as fp :
        lines = fp.read().split('\n')
    return lines[5].split()

def get_poscar_natoms (fname) :
    with open(fname, 'r') as fp :
        lines = fp.read().split('\n')
    return [int(ii) for ii in lines[6].split()]

def _poscar_natoms(lines) :
    numb_atoms = 0
    for ii in lines[6].split() :
        numb_atoms += int(ii)
    return numb_atoms

def _poscar_scale_direct (str_in, scale) :
    lines = str_in.copy()
    numb_atoms = _poscar_natoms(lines)
    pscale = float(lines[1])
    pscale = pscale * scale
    lines[1] = str(pscale) + "\n"
    return lines

def _poscar_scale_cartesian (str_in, scale) :
    lines = str_in.copy()
    numb_atoms = _poscar_natoms(lines)
    # scale box
    for ii in range(2,5) :
        boxl = lines[ii].split()
        boxv = [float(ii) for ii in boxl]
        boxv = np.array(boxv) * scale
        lines[ii] = "%.16e %.16e %.16e\n" % (boxv[0], boxv[1], boxv[2])
    # scale coord
    for ii in range(8, 8+numb_atoms) :
        cl = lines[ii].split()
        cv = [float(ii) for ii in cl]
        cv = np.array(cv) * scale
        lines[ii] = "%.16e %.16e %.16e\n" % (cv[0], cv[1], cv[2])
    return lines

def poscar_natoms(poscar_in) :
    with open(poscar_in, 'r') as fin :
        lines = list(fin)
    return _poscar_natoms(lines)

def poscar_scale (poscar_in, poscar_out, scale) :
    with open(poscar_in, 'r') as fin :
        lines = list(fin)
    if 'D' == lines[7][0] or 'd' == lines[7][0] :
        lines = _poscar_scale_direct(lines, scale)
    elif 'C' == lines[7][0] or 'c' == lines[7][0] :
        lines = _poscar_scale_cartesian(lines, scale)
    else :
        raise RuntimeError("Unknow poscar coord style at line 7: %s" % lines[7])
    with open(poscar_out, 'w') as fout:
        fout.write("".join(lines))

def poscar_vol (poscar_in) :
    with open(poscar_in, 'r') as fin :
        lines = list(fin)
    box = []
    for ii in range(2,5) :
        words = lines[ii].split()
        vec = [float(jj) for jj in words]
        box.append(vec)
    scale = float(lines[1].split()[0])
    box = np.array(box)
    box *= scale
    return np.linalg.det(box)

def _make_vasp_kp_gamma(kpoints):
    ret = ""
    ret += "Automatic mesh\n"
    ret += "0\n"
    ret += "Gamma\n"
    ret += "%d %d %d\n" % (kpoints[0], kpoints[1], kpoints[2])
    ret += "0  0  0\n"
    return ret

def _make_vasp_kp_mp(kpoints):
    ret = ""
    ret += "K-Points\n"
    ret += " 0\n"
    ret += "Monkhorst Pack\n"
    ret += "%d %d %d\n" % (kpoints[0], kpoints[1], kpoints[2])
    ret += " 0  0  0\n"
    return ret

def make_vasp_kpoints (kpoints, kgamma = False) :
    if kgamma :
        ret = _make_vasp_kp_gamma(kpoints)
    else :
        ret = _make_vasp_kp_mp(kpoints)
    return ret


def make_vasp_kpoints_from_incar(work_dir,jdata):
    cwd=os.getcwd()
    fp_aniso_kspacing = jdata.get('fp_aniso_kspacing')
    os.chdir(work_dir)
    # get kspacing and kgamma from incar
    assert(os.path.exists('INCAR'))
    with open('INCAR') as fp:
        incar = fp.read()
    standard_incar = incar_upper(Incar.from_string(incar))
    if fp_aniso_kspacing is None:
        try:
            kspacing = standard_incar['KSPACING']
        except KeyError:
            raise RuntimeError ("KSPACING must be given in INCAR")
    else :
        kspacing = fp_aniso_kspacing
    try:
        gamma = standard_incar['KGAMMA']
        if isinstance(gamma,bool):
            pass
        else:
            if gamma[0].upper()=="T":
                gamma=True
            else:
                gamma=False
    except KeyError:
        raise RuntimeError ("KGAMMA must be given in INCAR")
    # check poscar
    assert(os.path.exists('POSCAR'))
    # make kpoints
    ret=make_kspacing_kpoints('POSCAR', kspacing, gamma)
    kp=Kpoints.from_string(ret)
    kp.write_file("KPOINTS")
    os.chdir(cwd)



#!/usr/bin/env python3 

import time
import os,json,shutil,re,glob,argparse
import numpy as np
import subprocess as sp
import dpgen.data.tools.hcp as hcp
import dpgen.data.tools.fcc as fcc
import dpgen.data.tools.diamond as diamond
import dpgen.data.tools.sc as sc
import dpgen.data.tools.bcc as bcc
from dpgen import dlog
from dpgen import ROOT_PATH
from dpgen.remote.decide_machine import  decide_fp_machine
from dpgen.dispatcher.Dispatcher import Dispatcher, make_dispatcher
#-----PMG---------
from pymatgen.io.vasp import Poscar
from pymatgen import Structure,Element
from pymatgen.io.ase import AseAtomsAdaptor
#-----ASE-------
from ase.io import read
from ase.build import general_surface


def create_path (path) :
    path += '/'
    if os.path.isdir(path) : 
        dirname = os.path.dirname(path)        
        counter = 0
        while True :
            bk_dirname = dirname + ".bk%03d" % counter
            if not os.path.isdir(bk_dirname) : 
                shutil.move (dirname, bk_dirname) 
                break
            counter += 1
    os.makedirs (path)
    return path

def replace (file_name, pattern, subst) :
    file_handel = open (file_name, 'r')
    file_string = file_handel.read ()
    file_handel.close ()
    file_string = ( re.sub (pattern, subst, file_string) )
    file_handel = open (file_name, 'w')
    file_handel.write (file_string)
    file_handel.close ()

"""
1       make unit cell
        copy to make super cell
        place element
        make vasp relax
1a      vasp relax
2       scale system
        perturb system
3       make vasp md
3a      vasp md
4       collect md data
"""
global_dirname_02 = '00.place_ele'
global_dirname_03 = '01.scale_pert'
global_dirname_04 = '02.md'

max_layer_numb = 50

def out_dir_name(jdata) :
    super_cell = jdata['super_cell']    

    from_poscar= jdata.get('from_poscar',False)

    if  from_poscar:
        from_poscar_path = jdata['from_poscar_path']
        poscar_name = os.path.basename(from_poscar_path)
        cell_str = "%02d" % (super_cell[0])
        for ii in range(1,len(super_cell)) :
            cell_str = cell_str + ("x%02d" % super_cell[ii])
        return poscar_name + '.' + cell_str
    else:
        cell_type = jdata['cell_type']
        elements = jdata['elements']
        super_cell = jdata['super_cell']    

        ele_str = "surf."
        for ii in elements:
            ele_str = ele_str + ii.lower()
        cell_str = "%02d" % (super_cell[0])
        for ii in range(1,len(super_cell)) :
            cell_str = cell_str + ("x%02d" % super_cell[ii])
        return ele_str + '.' + cell_type + '.' + cell_str

def class_cell_type(jdata) :
    ct = jdata['cell_type']
    if ct == "hcp" :
        cell_type = hcp
    elif ct == "fcc" :
        cell_type = fcc
    elif ct == "diamond" :
        cell_type = diamond
    elif ct == "sc" :
        cell_type = sc
    elif ct == "bcc" :
        cell_type = bcc
    else :
        raise RuntimeError("unknow cell type %s" % ct)
    return cell_type

def poscar_ele(poscar_in, poscar_out, eles, natoms) :
    ele_line = ""
    natom_line = ""
    for ii in eles :
        ele_line += str(ii) + " "
    for ii in natoms :
        natom_line += str(ii) + " "
    with open(poscar_in, 'r') as fin :
        lines = list(fin)
        lines[5] = ele_line + "\n"
        lines[6] = natom_line + "\n"
    with open(poscar_out, 'w') as fout :
        fout.write("".join(lines))

def _poscar_natoms(lines) :
    numb_atoms = 0
    for ii in lines[6].split() :
        numb_atoms += int(ii)
    return numb_atoms

def poscar_natoms(poscar_in) :
    with open(poscar_in, 'r') as fin :
        lines = list(fin)
        return _poscar_natoms(lines)

def poscar_shuffle(poscar_in, poscar_out) :
    with open(poscar_in, 'r') as fin :
        lines = list(fin)
    numb_atoms = _poscar_natoms(lines)
    idx = np.arange(8, 8+numb_atoms)
    np.random.shuffle(idx)
    out_lines = lines[0:8]
    for ii in range(numb_atoms) :
        out_lines.append(lines[idx[ii]])
    with open(poscar_out, 'w') as fout:
        fout.write("".join(out_lines))

def poscar_scale_direct (str_in, scale) :
    lines = str_in.copy()
    numb_atoms = _poscar_natoms(lines)
    pscale = float(lines[1])
    pscale = pscale * scale
    lines[1] = str(pscale) + "\n"
    return lines

def poscar_scale_cartesian (str_in, scale) :
    lines = str_in.copy()
    numb_atoms = _poscar_natoms(lines)
    # scale box
    for ii in range(2,5) :
        boxl = lines[ii].split()
        boxv = [float(ii) for ii in boxl]
        boxv = np.array(boxv) * scale
        lines[ii] = "%.16e %.16e %.16e\n" % (boxv[0], boxv[1], boxv[2])
    # scale coord
    for ii in range(8, 8+numb_atoms) :
        cl = lines[ii].split()
        cv = [float(ii) for ii in cl]
        cv = np.array(cv) * scale
        lines[ii] = "%.16e %.16e %.16e\n" % (cv[0], cv[1], cv[2])
    return lines    

def poscar_scale (poscar_in, poscar_out, scale) :
    with open(poscar_in, 'r') as fin :
        lines = list(fin)
    if 'D' == lines[7][0] or 'd' == lines[7][0]: 
        lines = poscar_scale_direct(lines, scale)
    elif 'C' == lines[7][0] or 'c' == lines[7][0] :
        lines = poscar_scale_cartesian(lines, scale)
    else :
        raise RuntimeError("Unknow poscar style at line 7: %s" % lines[7])
    with open(poscar_out, 'w') as fout:
        fout.write("".join(lines))

def poscar_elong (poscar_in, poscar_out, elong, shift_center=True) :
    with open(poscar_in, 'r') as fin :
        lines = list(fin)
    if lines[7][0].upper() != 'C' :
        raise RuntimeError("only works for Cartesian POSCAR")
    sboxz = lines[4].split()
    boxz = np.array([float(sboxz[0]), float(sboxz[1]), float(sboxz[2])])
    boxzl = np.linalg.norm(boxz)
    elong_ratio = elong / boxzl
    boxz = boxz * (1. + elong_ratio)
    lines[4] = '%.16e %.16e %.16e\n' % (boxz[0],boxz[1],boxz[2])
    if shift_center:
       poscar_str="".join(lines)
       st=Structure.from_str(poscar_str,fmt='poscar')
       cart_coords=st.cart_coords
       z_mean=cart_coords[:,2].mean()
       z_shift=st.lattice.c/2-z_mean
       cart_coords[:,2]=cart_coords[:,2]+z_shift
       nst=Structure(st.lattice,st.species,coords=cart_coords,coords_are_cartesian=True)
       nst.to('poscar',poscar_out)
    else:
       with open(poscar_out, 'w') as fout:
            fout.write("".join(lines))

def make_unit_cell (jdata) :

    from_poscar= jdata.get('from_poscar',False)
    if not from_poscar:
       latt = jdata['latt']
       cell_type = class_cell_type(jdata)

    out_dir = jdata['out_dir']
    path_uc = os.path.join(out_dir, global_dirname_02)

    cwd = os.getcwd()    
    # for ii in scale :
    # path_work = create_path(os.path.join(path_uc, '%.3f' % ii))
    path_work = create_path(path_uc)    
    os.chdir(path_work)
    if not from_poscar:
       with open('POSCAR.unit', 'w') as fp:
           fp.write (cell_type.poscar_unit(latt))
    os.chdir(cwd)        

def make_super_cell_pymatgen (jdata) :

    make_unit_cell(jdata)
    out_dir = jdata['out_dir']
    path_uc = os.path.join(out_dir, global_dirname_02)
    
    elements=[Element(ii) for ii in jdata['elements']]
    if 'vacuum_min' in jdata:
        vacuum_min=jdata['vacuum_min']
    else:
        vacuum_min=max([float(ii.atomic_radius) for ii in elements])

    from_poscar= jdata.get('from_poscar',False)

    if from_poscar:
        from_poscar_path = jdata['from_poscar_path']
        poscar_name = os.path.basename(from_poscar_path)
        ss = Structure.from_file(poscar_name)
    else:
        from_path = path_uc
        from_file = os.path.join(from_path, 'POSCAR.unit')
        ss = Structure.from_file(from_file)
        # ase only support X type  element
        for i in range(len(ss)):
            ss[i]='X'

    ss=AseAtomsAdaptor.get_atoms(ss)

    all_millers = jdata['millers']
    path_sc = os.path.join(out_dir, global_dirname_02)
    
    user_layer_numb = None # set default value
    z_min = None  
    if 'layer_numb' in jdata:
        user_layer_numb = jdata['layer_numb']
    else:
        z_min = jdata['z_min']

    super_cell = jdata['super_cell']

    cwd = os.getcwd()    
    path_work = (path_sc)    
    path_work = os.path.abspath(path_work)
    os.chdir(path_work)
    for miller in all_millers:
        miller_str=""
        for ii in miller :
            miller_str += str(ii)        
        path_cur_surf = create_path('surf-'+miller_str)
        os.chdir(path_cur_surf)
        #slabgen = SlabGenerator(ss, miller, z_min, 1e-3)
        if user_layer_numb:
            slab=general_surface.surface(ss,indices=miller,vacuum=vacuum_min,layers=user_layer_numb)
        else:
           # build slab according to z_min value   
           for layer_numb in range( 1,max_layer_numb+1):
               slab=general_surface.surface(ss,indices=miller,vacuum=vacuum_min,layers=layer_numb)
               if slab.cell.lengths()[-1] >= z_min:
                  break
               if layer_numb == max_layer_numb:
                  raise RuntimeError("can't build the required slab")
        #all_slabs = slabgen.get_slabs() 
        dlog.info(os.getcwd())
        #dlog.info("Miller %s: The slab has %s termination, use the first one" %(str(miller), len(all_slabs)))
        #all_slabs[0].to('POSCAR', 'POSCAR')
        slab.write('POSCAR',vasp5=True)
        if super_cell[0] > 1 or super_cell[1] > 1 :
            st=Structure.from_file('POSCAR')
            st.make_supercell([super_cell[0], super_cell[1], 1])
            st.to('POSCAR','POSCAR')
        os.chdir(path_work)
    os.chdir(cwd)        

def make_combines (dim, natoms) :
    if dim == 1 :
        return [[natoms]]
    else :
        res = []
        for ii in range(natoms+1) :
            rest = natoms - ii
            tmp_combines = make_combines(dim-1, rest)
            for jj in tmp_combines :
                jj.append(ii)
            if len(res) == 0 :
                res = tmp_combines
            else :
                res += tmp_combines
        return res

def place_element (jdata) :
    out_dir = jdata['out_dir']
    super_cell = jdata['super_cell']
    cell_type = class_cell_type(jdata)
    elements = jdata['elements']
    from_poscar= jdata.get('from_poscar',False)
    path_sc = os.path.join(out_dir, global_dirname_02)
    path_pe = os.path.join(out_dir, global_dirname_02)    
    path_sc = os.path.abspath(path_sc)
    path_pe = os.path.abspath(path_pe)
    
    assert(os.path.isdir(path_sc))
    assert(os.path.isdir(path_pe))
    cwd = os.getcwd()
    os.chdir(path_sc)
    surf_list = glob.glob('surf-*')
    surf_list.sort()
    os.chdir(cwd)

    for ss in surf_list:
        path_surf = os.path.join(path_sc, ss)        
        pos_in = os.path.join(path_surf, 'POSCAR')
        natoms = poscar_natoms(pos_in)
        combines = np.array(make_combines(len(elements), natoms), dtype = int)
        for ii in combines :
            if any(ii == 0) :
                continue
            comb_name = "sys-"
            for idx,jj in enumerate(ii) :            
                comb_name += "%04d" % jj
                if idx != len(ii)-1 :
                    comb_name += "-"
            path_work = os.path.join(path_surf, comb_name)
            create_path(path_work)
            pos_out = os.path.join(path_work, 'POSCAR')
            if from_poscar:
               shutil.copy2( pos_in, pos_out) 
            else:
               poscar_ele(pos_in, pos_out, elements, ii)
            poscar_shuffle(pos_out, pos_out)

def make_vasp_relax (jdata) :
    out_dir = jdata['out_dir']
    potcars = jdata['potcars']
    cwd = os.getcwd()

    work_dir = os.path.join(out_dir, global_dirname_02)
    assert (os.path.isdir(work_dir))
    work_dir = os.path.abspath(work_dir)
    if os.path.isfile(os.path.join(work_dir, 'INCAR' )) :
        os.remove(os.path.join(work_dir, 'INCAR' ))
    if os.path.isfile(os.path.join(work_dir, 'POTCAR')) :
        os.remove(os.path.join(work_dir, 'POTCAR'))
    shutil.copy2( jdata['relax_incar'], 
                 os.path.join(work_dir, 'INCAR'))
    out_potcar = os.path.join(work_dir, 'POTCAR')
    with open(out_potcar, 'w') as outfile:
        for fname in potcars:
            with open(fname) as infile:
                outfile.write(infile.read())
    
    os.chdir(work_dir)
    
    sys_list = glob.glob(os.path.join('surf-*', 'sys-*'))
    for ss in sys_list:
        os.chdir(ss)
        ln_src = os.path.relpath(os.path.join(work_dir,'INCAR'))
        os.symlink(ln_src, 'INCAR')
        ln_src = os.path.relpath(os.path.join(work_dir,'POTCAR'))
        os.symlink(ln_src, 'POTCAR')
        os.chdir(work_dir)
    os.chdir(cwd)

def poscar_scale_direct (str_in, scale) :
    lines = str_in.copy()
    numb_atoms = _poscar_natoms(lines)
    pscale = float(lines[1])
    pscale = pscale * scale
    lines[1] = str(pscale) + "\n"
    return lines

def poscar_scale_cartesian (str_in, scale) :
    lines = str_in.copy()
    numb_atoms = _poscar_natoms(lines)
    # scale box
    for ii in range(2,5) :
        boxl = lines[ii].split()
        boxv = [float(ii) for ii in boxl]
        boxv = np.array(boxv) * scale
        lines[ii] = "%.16e %.16e %.16e\n" % (boxv[0], boxv[1], boxv[2])
    # scale coord
    for ii in range(8, 8+numb_atoms) :
        cl = lines[ii].split()
        cv = [float(ii) for ii in cl]
        cv = np.array(cv) * scale
        lines[ii] = "%.16e %.16e %.16e\n" % (cv[0], cv[1], cv[2])
    return lines

def poscar_scale (poscar_in, poscar_out, scale) :
    with open(poscar_in, 'r') as fin :
        lines = list(fin)
    if 'D' == lines[7][0] or 'd' == lines[7][0]:
        lines = poscar_scale_direct(lines, scale)
    elif 'C' == lines[7][0] or 'c' == lines[7][0] :
        lines = poscar_scale_cartesian(lines, scale)
    else :
        raise RuntimeError("Unknow poscar style at line 7: %s" % lines[7])

    poscar=Poscar.from_string("".join(lines))
    with open(poscar_out, 'w') as fout:
        fout.write(poscar.get_string(direct=False))

def make_scale(jdata):
    out_dir = jdata['out_dir']
    scale = jdata['scale']    
    skip_relax = jdata['skip_relax']    

    cwd = os.getcwd()
    init_path = os.path.join(out_dir, global_dirname_02)
    init_path = os.path.abspath(init_path)
    work_path = os.path.join(out_dir, global_dirname_03)
    os.chdir(init_path)
    init_sys = glob.glob(os.path.join('surf-*', 'sys-*'))
    init_sys.sort()
    os.chdir(cwd)

    create_path(work_path)
    for ii in init_sys :
        for jj in scale :
            if skip_relax :
                pos_src = os.path.join(os.path.join(init_path, ii), 'POSCAR')
                assert(os.path.isfile(pos_src))
            else :
                try:
                    pos_src = os.path.join(os.path.join(init_path, ii), 'CONTCAR')
                    assert(os.path.isfile(pos_src))
                except:
                    raise RuntimeError("not file %s, vasp relaxation should be run before scale poscar")
            scale_path = os.path.join(work_path, ii)
            scale_path = os.path.join(scale_path, "scale-%.3f" % jj)
            create_path(scale_path)
            os.chdir(scale_path)
            poscar_scale(pos_src, 'POSCAR', jj)
            os.chdir(cwd)

def pert_scaled(jdata) :
    out_dir = jdata['out_dir']
    scale = jdata['scale']    
    pert_box = jdata['pert_box']
    pert_atom = jdata['pert_atom']
    pert_numb = jdata['pert_numb']
    vacuum_max = jdata['vacuum_max']
    vacuum_resol = jdata.get('vacuum_resol',[])
    if vacuum_resol:
       if len(vacuum_resol)==1:
          elongs = np.arange(vacuum_resol[0], vacuum_max, vacuum_resol[0])
       elif len(vacuum_resol)==2:
          mid_point = jdata.get('mid_point')
          head_elongs = np.arange(vacuum_resol[0], mid_point, vacuum_resol[0]).tolist()
          tail_elongs = np.arange(mid_point, vacuum_max, vacuum_resol[1]).tolist()
          elongs = np.unique(head_elongs+tail_elongs).tolist()
       else:
          raise RuntimeError("the length of vacuum_resol must equal 1 or 2")
          
    else:         
       vacuum_num = jdata['vacuum_numb']
       head_ratio = jdata['head_ratio']
       mid_point = jdata['mid_point']
       head_numb  = int(vacuum_num*head_ratio)
       tail_numb = vacuum_num - head_numb
       head_elongs = np.linspace(0,mid_point,head_numb).tolist()
       tail_elongs = np.linspace(mid_point,vacuum_max,tail_numb+1).tolist()
       elongs = np.unique(head_elongs+tail_elongs).tolist()
    
    cwd = os.getcwd()
    path_sp = os.path.join(out_dir, global_dirname_03)
    assert(os.path.isdir(path_sp))
    path_sp = os.path.abspath(path_sp)
    os.chdir(path_sp)
    sys_pe = glob.glob(os.path.join('surf-*', 'sys-*'))
    sys_pe.sort()
    os.chdir(cwd)    

    pert_cmd = "python "+os.path.join(ROOT_PATH, 'data/tools/create_random_disturb.py')
    pert_cmd += ' -etmax %f -ofmt vasp POSCAR %d %f > /dev/null' %(pert_box, pert_numb, pert_atom)    
    for ii in sys_pe :
        for jj in scale :
            path_scale = path_sp
            path_scale = os.path.join(path_scale, ii)
            path_scale = os.path.join(path_scale, 'scale-%.3f' % jj)
            assert(os.path.isdir(path_scale))
            os.chdir(path_scale)
            dlog.info(os.getcwd())
            poscar_in = os.path.join(path_scale, 'POSCAR')
            assert(os.path.isfile(poscar_in))
            for ll in elongs:
                path_elong = path_scale
                path_elong = os.path.join(path_elong, 'elong-%3.3f' % ll) 
                create_path(path_elong)
                os.chdir(path_elong)
                poscar_elong(poscar_in, 'POSCAR', ll)                
                sp.check_call(pert_cmd, shell = True)
                for kk in range(pert_numb) :
                    pos_in = 'POSCAR%d.vasp' % (kk+1)
                    dir_out = '%06d' % (kk+1)
                    create_path(dir_out)
                    pos_out = os.path.join(dir_out, 'POSCAR')
                    poscar_shuffle(pos_in, pos_out)
                    os.remove(pos_in)
                kk = -1
                pos_in = 'POSCAR'
                dir_out = '%06d' % (kk+1)
                create_path(dir_out)
                pos_out = os.path.join(dir_out, 'POSCAR')
                poscar_shuffle(pos_in, pos_out)
                os.chdir(cwd)
def _vasp_check_fin (ii) :
    if os.path.isfile(os.path.join(ii, 'OUTCAR')) :
        with open(os.path.join(ii, 'OUTCAR'), 'r') as fp :
            content = fp.read()
            count = content.count('Elapse')
            if count != 1 :
                return False
    else :
        return False
    return True

def run_vasp_relax(jdata, mdata, dispatcher):
    fp_command = mdata['fp_command']
    fp_group_size = mdata['fp_group_size']
    fp_resources = mdata['fp_resources']
    machine_type = mdata['fp_machine']['machine_type']
    work_dir = os.path.join(jdata['out_dir'], global_dirname_02)
    
    forward_files = ["POSCAR", "INCAR", "POTCAR"]
    backward_files = ["OUTCAR","CONTCAR"]
    forward_common_files = []
    #if 'cvasp' in mdata['fp_resources']:
    #    if mdata['fp_resources']['cvasp']:
    #        forward_common_files=['cvasp.py']
    relax_tasks = glob.glob(os.path.join(work_dir, "surf-*/","sys-*"))
    relax_tasks.sort()
    #dlog.info("work_dir",work_dir)
    #dlog.info("relax_tasks",relax_tasks)
    if len(relax_tasks) == 0:
        return

    relax_run_tasks = []
    for ii in relax_tasks : 
        if not _vasp_check_fin(ii):
            relax_run_tasks.append(ii)
    run_tasks = [ii.replace(work_dir+"/", "") for ii in relax_run_tasks]

    #dlog.info(run_tasks)
    dispatcher.run_jobs(fp_resources,
                       [fp_command],
                       work_dir,
                       run_tasks,
                       fp_group_size,
                       forward_common_files,
                       forward_files,
                       backward_files)

def gen_init_surf(args):
    try:
       import ruamel
       from monty.serialization import loadfn,dumpfn
       warnings.simplefilter('ignore', ruamel.yaml.error.MantissaNoDotYAML1_1Warning)
       jdata=loadfn(args.PARAM)
       if args.MACHINE is not None:
          mdata=loadfn(args.MACHINE)
    except:
        with open (args.PARAM, 'r') as fp :
            jdata = json.load (fp)
        if args.MACHINE is not None:
           with open (args.MACHINE, "r") as fp:
                mdata = json.load(fp)

    out_dir = out_dir_name(jdata)
    jdata['out_dir'] = out_dir
    dlog.info ("# working dir %s" % out_dir)
    
    if args.MACHINE is not None:
       # Decide a proper machine
       mdata = decide_fp_machine(mdata)
       disp = make_dispatcher(mdata["fp_machine"])

    #stage = args.STAGE
    stage_list = [int(i) for i in jdata['stages']]
    for stage in stage_list:
        if stage == 1 :
            create_path(out_dir)
            make_super_cell_pymatgen(jdata)
            place_element(jdata)
            make_vasp_relax(jdata)
            if args.MACHINE is not None:
               run_vasp_relax(jdata, mdata, disp)
        elif stage == 2 :
            make_scale(jdata)
            pert_scaled(jdata)
        else :
            raise RuntimeError("unknown stage %d" % stage)
    
if __name__ == "__main__":
   parser = argparse.ArgumentParser(
       description="Generating initial data for surface systems.")
   parser.add_argument('PARAM', type=str, 
                       help="parameter file, json/yaml format")
   parser.add_argument('MACHINE', type=str,default=None,nargs="?",
                       help="machine file, json/yaml format")
   args = parser.parse_args()
   gen_init_surf(args)



#!/usr/bin/env python3

import os, re, argparse, filecmp, json, glob
import subprocess as sp
import numpy as np
import dpgen.auto_test.lib.vasp as vasp
import dpgen.auto_test.lib.lammps as lammps
from phonopy import Phonopy
from phonopy.structure.atoms import PhonopyAtoms

import yaml
import phonopy




global_equi_name = '00.equi'
global_task_name = '06.phonon'

'''
link poscar
link potcar
make incar
'''

def get_force_from_dump(cell):
    na=len(cell)
    with open("dump.relax","r") as fp:
        lines = fp.readlines()
        flag=False
        index=[]
        forces=[]
        for line in lines:
            if flag:
                data=line.split()
                index.append(data[0])
                forces.append(data[5:9])
            if len(forces)==na:
                break
            if 'fx fy fz' in line:
                flag=True
        index = np.asarray(index, int)
        indexing = np.argsort(index)
        if len(forces)==na:
            forces=np.asarray(np.reshape(forces,(na,3)),float)[indexing, :]
        else:
            raise RuntimeError('Incomplete result: dump.relax')
    return forces

def cmpt_vasp(jdata, conf_dir) :
    fp_params = jdata['vasp_params']
    kspacing = fp_params['kspacing']
    supercell_matrix=jdata['supercell_matrix']
    if 'relax_incar' in jdata.keys():
        vasp_str='vasp-relax_incar'
    else:
        vasp_str='vasp-k%.2f' % kspacing

    conf_path = os.path.abspath(conf_dir)
    task_path = re.sub('confs', global_task_name, conf_path)
    task_path = os.path.join(task_path, vasp_str)
    cwd = os.getcwd()
    os.chdir(task_path)
    if os.path.isfile('vasprun.xml'):
        os.system('phonopy --fc vasprun.xml')
        if os.path.isfile('FORCE_CONSTANTS'):
            os.system('phonopy --dim="%d %d %d" -c POSCAR-unitcell band.conf'%(supercell_matrix[0],supercell_matrix[1],supercell_matrix[2]))
            os.system('phonopy-bandplot --gnuplot band.yaml > band.dat')
            print('band.dat is created')
        else:
            print('FORCE_CONSTANTS No such file')
    else:
        print('vasprun.xml No such file')

    
    
def cmpt_lammps(jdata, conf_dir, task_type) :
    supercell_matrix=jdata['supercell_matrix']

    conf_path = os.path.abspath(conf_dir)
    task_path = re.sub('confs', global_task_name, conf_path)
    task_path = os.path.join(task_path, task_type)
    task_poscar = os.path.join(task_path, 'POSCAR')

    os.chdir(task_path)
    if os.path.isfile('FORCE_CONSTANTS'):
        os.system('phonopy --dim="%d %d %d" -c POSCAR band.conf'%(supercell_matrix[0],supercell_matrix[1],supercell_matrix[2]))
        os.system('phonopy-bandplot --gnuplot band.yaml > band.dat')
    else:
        print('FORCE_CONSTANTS No such file')


def _main() :
    parser = argparse.ArgumentParser(
        description="cmpt 06.phonon")
    parser.add_argument('TASK', type=str,
                        help='the task of generation, vasp or lammps')
    parser.add_argument('PARAM', type=str,
                        help='json parameter file')
    parser.add_argument('CONF', type=str,
                        help='the path to conf')
    args = parser.parse_args()

    with open (args.PARAM, 'r') as fp :
        jdata = json.load (fp)

#    print('generate %s task with conf %s' % (args.TASK, args.CONF))
    if args.TASK == 'vasp':
        cmpt_vasp(jdata, args.CONF)               
    elif args.TASK == 'deepmd' or args.TASK =='meam' :
        cmpt_lammps(jdata, args.CONF,args.TASK)
    else :
        raise RuntimeError("unknow task ", args.TASK)
    
if __name__ == '__main__' :
    _main()




import os,sys,json,glob,shutil
import dpdata
import numpy as np
import unittest

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))
__package__ = 'generator'
from .context import sys_link_fp_vasp_pp
from .context import make_iter_name
from .context import fp_name
from .context import setUpModule

class TestConcatVASPPP(unittest.TestCase):
    def test(self):
        iter_index = 0
        iter_name = make_iter_name(iter_index)
        work_path = os.path.join(iter_name, fp_name)

        if os.path.isdir(iter_name) :
            shutil.rmtree(iter_name)
        os.makedirs(iter_name, exist_ok = False)
        os.makedirs(work_path, exist_ok = False)
        os.makedirs(os.path.join(work_path, 'task.000.000000'), exist_ok = False)
        os.makedirs(os.path.join(work_path, 'task.000.000001'), exist_ok = False)
        os.makedirs(os.path.join(work_path, 'task.001.000000'), exist_ok = False)
        os.makedirs(os.path.join(work_path, 'task.001.000001'), exist_ok = False)
        shutil.copyfile(os.path.join('vasp', 'POSCAR.oh'), 
                        os.path.join(work_path, 'task.000.000000', 'POSCAR'))
        shutil.copyfile(os.path.join('vasp', 'POSCAR.oh'), 
                        os.path.join(work_path, 'task.000.000001', 'POSCAR'))
        shutil.copyfile(os.path.join('vasp', 'POSCAR.ch4'), 
                        os.path.join(work_path, 'task.001.000000', 'POSCAR'))
        shutil.copyfile(os.path.join('vasp', 'POSCAR.ch4'), 
                        os.path.join(work_path, 'task.001.000001', 'POSCAR'))
        sys_link_fp_vasp_pp(0, {'type_map' : ['H', 'C', 'O'], 
                                'fp_pp_path': os.path.join('vasp', 'potcars'), 
                                'fp_pp_files': ['POTCAR.H', 'POTCAR.C', 'POTCAR.O'],
        })
        self.assertTrue(os.path.isfile(os.path.join(work_path, 'POTCAR.000')))
        self.assertTrue(os.path.isfile(os.path.join(work_path, 'POTCAR.001')))        
        with open((os.path.join(work_path, 'POTCAR.000'))) as fp:
            pot = fp.read()
            self.assertEqual(pot, 'O\nH\n')
        with open((os.path.join(work_path, 'POTCAR.001'))) as fp:
            pot = fp.read()
            self.assertEqual(pot, 'H\nC\n')
        for ii in ['task.000.000000', 'task.000.000001'] :
            with open(os.path.join(work_path, ii, 'POTCAR')) as fp:
                pot = fp.read()
                self.assertEqual(pot, 'O\nH\n')
        for ii in ['task.001.000000', 'task.001.000001'] :
            with open(os.path.join(work_path, ii, 'POTCAR')) as fp:
                pot = fp.read()
                self.assertEqual(pot, 'H\nC\n')
                


if __name__ == '__main__':
    unittest.main()





import sys,os
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..','..')))
import dpgen
from dpgen.database.entry import Entry
from dpgen.database.run import parsing_vasp
from dpgen.database.vasp import VaspInput,DPPotcar

def setUpModule():
    os.chdir(os.path.abspath(os.path.dirname(__file__)))



import os,sys,json,glob,shutil
import dpdata
import numpy as np
import unittest

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))
__package__ = 'auto_test'
from .context import make_kspacing_kpoints
from .context import setUpModule

from pymatgen.io.vasp import Incar
from dpgen.auto_test.gen_02_elastic import make_vasp

class Test02(unittest.TestCase):
    def tearDown(self):
        if os.path.exists('02.elastic'):
            shutil.rmtree('02.elastic')

    def test_make_vasp (self):
        jdata = {
            'relax_incar' : 'vasp_input/INCAR.rlx',
            'potcar_map': {'Si': 'vasp_input/POTCAR' },
        }
        make_vasp(jdata, 'confs/si/mp-149')
        
        target_path = '02.elastic/si/mp-149/vasp-relax_incar'
        equi_path = '00.equi/si/mp-149/vasp-relax_incar'
        dfm_dirs = glob.glob(os.path.join(target_path, 'dfm*'))
       
        # check root INCAR
        incar0 = Incar.from_file(os.path.join('vasp_input', 'INCAR.rlx'))
        incar1 = Incar.from_file(os.path.join(target_path, 'INCAR'))
        self.assertFalse(incar0 == incar1)
        incar0['ISIF'] = 2
        self.assertTrue(incar0 == incar1)
        # check root POTCAR
        with open(os.path.join('vasp_input', 'POTCAR')) as fp:
            pot0 = fp.read()
        with open(os.path.join(target_path, 'POTCAR')) as fp:
            pot1 = fp.read()
        self.assertEqual(pot0, pot1)
        # check root POSCAR
        self.assertEqual(os.path.realpath(os.path.join(target_path, 'POSCAR')), 
                         os.path.realpath(os.path.join(equi_path, 'CONTCAR')))
        # check subdir
        for ii in dfm_dirs:
            self.assertEqual(os.path.realpath(os.path.join(ii, 'INCAR')),
                             os.path.realpath(os.path.join(target_path, 'INCAR')))
            self.assertEqual(os.path.realpath(os.path.join(ii, 'KPOINTS')),
                             os.path.realpath(os.path.join(target_path, 'KPOINTS')))
            self.assertEqual(os.path.realpath(os.path.join(ii, 'POTCAR')),
                             os.path.realpath(os.path.join(target_path, 'POTCAR')))



#!/usr/bin/env python3 

import glob, os, shutil

def copy(kspacing, static = False) :
    test_dir = os.path.abspath(os.path.dirname(__file__))
    if static :
        vasp_name = 'vasp-static-k%.2f' % kspacing
    else :
        vasp_name = 'vasp-k%.2f' % kspacing    
    meam_name = 'meam'
    confs = glob.glob(os.path.join(test_dir, "confs"))
    params = glob.glob(os.path.join(test_dir, "param.json"))
    cmpts = glob.glob(os.path.join(test_dir, "cmpt_*.py"))
    gens = glob.glob(os.path.join(test_dir, "gen_*.py"))
    vasp = glob.glob(os.path.join(test_dir, "0*/*/*/" + vasp_name))
    meam = glob.glob(os.path.join(test_dir, "0*/*/*/" + meam_name)) + \
           glob.glob(os.path.join(test_dir, "0*/*/*/" + meam_name + '-static')) + \
           glob.glob(os.path.join(test_dir, "0*/*/*/" + meam_name + '-reprod-k0.08')) 
    cwd = os.getcwd()
    os.chdir(test_dir)
    vasp_dirs = glob.glob( "0*/*/*/" + vasp_name)
    meam_dirs = glob.glob( "0*/*/*/" + meam_name) + \
                glob.glob( "0*/*/*/" + meam_name + '-static') + \
                glob.glob( "0*/*/*/" + meam_name + '-reprod-k0.16')
    os.chdir(cwd)
    for res,ii in zip(vasp,vasp_dirs) :
        dir_ii = os.path.dirname(ii)
        os.makedirs(dir_ii, exist_ok = True)
        os.chdir(dir_ii)
        if os.path.islink(vasp_name) :
            os.remove(vasp_name)
        os.symlink(os.path.relpath(res), vasp_name)
        os.chdir(cwd)

    for res,ii in zip(meam,meam_dirs) :
        dir_ii = os.path.dirname(ii)
        os.makedirs(dir_ii, exist_ok = True)
        os.chdir(dir_ii)
        tmp_name = os.path.basename(res)
        if os.path.islink(tmp_name) :
            os.remove(tmp_name)
        os.symlink(os.path.relpath(res), tmp_name)
        os.chdir(cwd)

    os.makedirs('results', exist_ok = True)
    for ii in cmpts :
        fname_ii = os.path.basename(ii)
        if os.path.islink(fname_ii) :
            os.remove(fname_ii)
        os.symlink(os.path.relpath(ii), fname_ii)
    for ii in gens :
        fname_ii = os.path.basename(ii)
        if os.path.islink(fname_ii) :
            os.remove(fname_ii)
        os.symlink(os.path.relpath(ii), fname_ii)
    for ii in confs :
        fname_ii = os.path.basename(ii)
        if os.path.islink(fname_ii) :
            os.remove(fname_ii)
        os.symlink(os.path.relpath(ii), fname_ii)
    for ii in params :
        fname_ii = os.path.basename(ii)
        if os.path.isfile(fname_ii) :
            os.remove(fname_ii)
        shutil.copy2(os.path.relpath(ii), fname_ii)

copy(0.08)
copy(0.08, static = True)
copy(0.16)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from os import path
from  dpgen import NAME,SHORT_CMD
import setuptools, datetime

readme_file = path.join(path.dirname(path.abspath(__file__)), 'README.md')
try:
    from m2r import parse_from_file
    readme = parse_from_file(readme_file)
except ImportError:
    with open(readme_file) as f:
        readme = f.read()

today = datetime.date.today().strftime("%b-%d-%Y")
with open(path.join('dpgen', '_date.py'), 'w') as fp :
    fp.write('date = \'%s\'' % today)

install_requires=['numpy>=1.14.3', 'dpdata>=0.1.14', 'pymatgen>=2019.1.13', 'ase', 'monty>2.0.0', 'paramiko', 'custodian']

setuptools.setup(
    name=NAME,
    use_scm_version={'write_to': 'dpgen/_version.py'},
    setup_requires=['setuptools_scm'],
    author="Han Wang",
    author_email="wang_han@iapcm.ac.cn",
    description="DPGen: The deep potential generator",
    long_description=readme,
    long_description_content_type="text/markdown",
    url="https://github.com/deepmodeling/dpgen",
    python_requires="~=3.6",
    packages=['dpgen', 
              'dpgen/generator',
              'dpgen/generator/lib',
              'dpgen/auto_test',
              'dpgen/auto_test/lib',
              'dpgen/data',
              'dpgen/data/tools',
              'dpgen/remote',
              'dpgen/dispatcher',
              'dpgen/database',
              'dpgen/tools',
              'dpgen/simplify',
              'dpgen/collect',
    ],
    # data_files = [('dpgen/tools/', ['dpgen/tools/update_time.sh', ])],
    # package_data={'example':['*.json']},
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)",
    ],
    keywords='deep potential generator active learning deepmd-kit',
    install_requires=install_requires,    
        entry_points={
          'console_scripts': [
              SHORT_CMD+'= dpgen.main:main']
   }
)




#!/usr/bin/env ovitos

from ovito.io import *
from ovito.modifiers import *

import numpy as np
import argparse

def copy_system (ncopy, fin, fout) :
    nx = ncopy[0]
    ny = ncopy[1]
    nz = ncopy[2]
    node = import_file(fin)
    pbc = ShowPeriodicImagesModifier(adjust_box = True,
                                     num_x = nx, 
                                     num_y = ny,
                                     num_z = nz, 
                                     replicate_x = True,
                                     replicate_y = True,
                                     replicate_z = True
    )
    node.modifiers.append(pbc)
    node.compute()
    export_file(node, fout, 'vasp')

parser = argparse.ArgumentParser(
    description="Copy system")
parser.add_argument('-n', '--ncopy', type=int, nargs = 3,
                    help="the number of copies in each direction")
parser.add_argument('INPUT', type=str,
                    help="the input file")
parser.add_argument('OUTPUT', type=str,
                    help="the output file")
args = parser.parse_args()

copy_system(args.ncopy, args.INPUT, args.OUTPUT)



"""
input: trajectory
00: ReaxFF MD (lammps)
01: build dataset (mddatasetbuilder)
02: fp (gaussian)
03: convert to deepmd data
output: data
"""

import argparse
import glob
import json
import os
import random

import dpdata
from dpgen import dlog
from dpgen.dispatcher.Dispatcher import make_dispatcher
from dpgen.generator.run import create_path, make_fp_task_name
from dpgen.util import sepline

reaxff_path = "00.reaxff"
build_path = "01.build"
fp_path = "02.fp"
data_path = "03.data"

trj_path = "lammpstrj"
ff_path = "ffield.reax"
data_init_path = "data.init"
control_path = "lmp_control"
lmp_path = "in.lmp"
dataset_name = "dpgen_init"


def link_reaxff(jdata):
    create_path(reaxff_path)
    task_path = os.path.join(reaxff_path, "task.000")
    create_path(task_path)

    rdata = jdata['reaxff']
    os.symlink(os.path.abspath(rdata["data"]), os.path.abspath(
        os.path.join(task_path, data_init_path)))
    os.symlink(os.path.abspath(rdata["ff"]), os.path.abspath(
        os.path.join(task_path, ff_path)))
    os.symlink(os.path.abspath(rdata["control"]), os.path.abspath(
        os.path.join(task_path, control_path)))
    with open(os.path.join(task_path, lmp_path), 'w') as f:
        f.write(make_lmp(jdata))


def make_lmp(jdata):
    rdata = jdata['reaxff']
    lmp_string = """units real
atom_style charge
read_data data.init
pair_style reax/c lmp_control
pair_coeff * * ffield.reax {type_map}
velocity all create {temp} {rand}
fix 1 all nvt temp {temp} {temp} {tau_t}
fix 2 all qeq/reax 1 0.0 10.0 1.0e-6 reax/c
dump 1 all custom {dump_freq} lammpstrj id type x y z 
timestep {dt}
run	{nstep}
""".format(
        type_map=" ".join(jdata['type_map']),
        temp=rdata['temp'],
        rand=random.randrange(1000000-1)+1,
        tau_t=rdata['tau_t'],
        dump_freq=rdata['dump_freq'],
        dt=rdata['dt'],
        nstep=rdata['nstep']
    )
    return lmp_string


def run_reaxff(jdata, mdata, dispatcher, log_file="reaxff_log"):
    work_path = reaxff_path
    reaxff_command = "{} -in {}".format(mdata["reaxff_command"], lmp_path)
    run_tasks = glob.glob(os.path.join(work_path, 'task.*'))
    run_tasks.sort()
    run_tasks = [os.path.basename(ii) for ii in run_tasks]

    dispatcher.run_jobs(mdata['reaxff_resources'],
                        [reaxff_command],
                        work_path,
                        run_tasks,
                        1,
                        [],
                        [ff_path, data_init_path, control_path, lmp_path],
                        [trj_path],
                        outlog=log_file,
                        errlog=log_file)


def link_trj(jdata):
    """link lammpstrj"""
    create_path(build_path)
    task_path = os.path.join(build_path, "task.000")
    create_path(task_path)

    os.symlink(os.path.abspath(os.path.join(reaxff_path, "task.000", trj_path)), os.path.abspath(
        os.path.join(task_path, trj_path)))


def run_build_dataset(jdata, mdata, dispatcher, log_file="build_log"):
    work_path = build_path
    build_command = "{cmd} -n {dataset_name} -a {type_map} -d {lammpstrj} -c {cutoff} -s {dataset_size} -k \"{qmkeywords}\" --nprocjob {nprocjob} --nproc {nproc}".format(
        cmd=mdata["build_command"],
        type_map=" ".join(jdata["type_map"]),
        lammpstrj=trj_path,
        cutoff=jdata["cutoff"],
        dataset_size=jdata["dataset_size"],
        qmkeywords=jdata["qmkeywords"],
        nprocjob=mdata["fp_resources"]["task_per_node"],
        nproc=mdata["build_resources"]["task_per_node"],
        dataset_name=dataset_name
    )
    run_tasks = glob.glob(os.path.join(work_path, 'task.*'))
    run_tasks.sort()
    run_tasks = [os.path.basename(ii) for ii in run_tasks]

    dispatcher.run_jobs(mdata['build_resources'],
                        [build_command],
                        work_path,
                        run_tasks,
                        1,
                        [],
                        [trj_path],
                        [f"dataset_{dataset_name}_gjf"],
                        outlog=log_file,
                        errlog=log_file)


def link_fp_input():
    all_input_file = glob.glob(os.path.join(
        build_path, "task.*", f"dataset_{dataset_name}_gjf", "*", "*.gjf"))
    work_path = fp_path
    create_path(work_path)

    for ii, fin in enumerate(all_input_file):
        dst_path = os.path.join(work_path, make_fp_task_name(0, ii))
        create_path(dst_path)
        os.symlink(os.path.abspath(fin), os.path.abspath(
            os.path.join(dst_path, "input")))


def run_fp(jdata,
           mdata,
           dispatcher,
           log_file="output",
           forward_common_files=[]):
    fp_command = mdata['fp_command']
    fp_group_size = mdata['fp_group_size']
    work_path = fp_path

    fp_tasks = glob.glob(os.path.join(work_path, 'task.*'))
    fp_tasks.sort()
    if len(fp_tasks) == 0:
        return

    fp_run_tasks = fp_tasks

    run_tasks = [os.path.basename(ii) for ii in fp_run_tasks]

    dispatcher.run_jobs(mdata['fp_resources'],
                        [fp_command],
                        work_path,
                        run_tasks,
                        fp_group_size,
                        [],
                        ["input"],
                        [log_file],
                        outlog=log_file,
                        errlog=log_file)


def convert_data(jdata):
    s = dpdata.MultiSystems(*[dpdata.LabeledSystem(x, fmt="gaussian/log")
                              for x in glob.glob(os.path.join(fp_path, "*", "output"))],
                            type_map=jdata["type_map"])
    s.to_deepmd_npy(data_path)
    dlog.info("Initial data is avaiable in %s" % os.path.abspath(data_path))


def gen_init_reaction(args):
    try:
        import ruamel
        from monty.serialization import loadfn, dumpfn
        warnings.simplefilter(
            'ignore', ruamel.yaml.error.MantissaNoDotYAML1_1Warning)
        jdata = loadfn(args.PARAM)
        if args.MACHINE is not None:
            mdata = loadfn(args.MACHINE)
    except:
        with open(args.PARAM, 'r') as fp:
            jdata = json.load(fp)
        if args.MACHINE is not None:
            with open(args.MACHINE, "r") as fp:
                mdata = json.load(fp)

    record = "record.reaction"
    iter_rec = -1
    numb_task = 7
    if os.path.isfile(record):
        with open(record) as frec:
            for line in frec:
                iter_rec = int(line.strip())
        dlog.info("continue from task %02d" % iter_rec)
    for ii in range(numb_task):
        sepline(str(ii), '-')
        if ii <= iter_rec:
            continue
        elif ii == 0:
            link_reaxff(jdata)
        elif ii == 1:
            dispatcher = make_dispatcher(mdata["reaxff_machine"])
            run_reaxff(jdata, mdata, dispatcher)
        elif ii == 2:
            link_trj(jdata)
        elif ii == 3:
            dispatcher = make_dispatcher(mdata["build_machine"])
            run_build_dataset(jdata, mdata, dispatcher)
        elif ii == 4:
            link_fp_input()
        elif ii == 5:
            dispatcher = make_dispatcher(mdata["fp_machine"])
            run_fp(jdata, mdata, dispatcher)
        elif ii == 6:
            convert_data(jdata)
        with open(record, "a") as frec:
            frec.write(str(ii)+'\n')



import os,sys,json,glob,shutil
import dpdata
import numpy as np
import unittest

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))
__package__ = 'data'
from .context import coll_vasp_md
from .context import out_dir_name
from .context import param_file
from .context import setUpModule

class TestCollVasp(unittest.TestCase):
    def setUp(self):
        with open (param_file, 'r') as fp :
            jdata = json.load (fp)
        self.odir = out_dir_name(jdata)
        assert os.path.isdir('out_data_02_md'), 'out data for post fp vasp should exist'
        if os.path.isdir(self.odir) :
            shutil.rmtree(self.odir)
        shutil.copytree('out_data_02_md', self.odir)
        self.ref_coord = [[[0, 0, 0], [2.3, 2.3, 2.3]],
                          [[0, 0, 0], [2.2, 2.3, 2.4]]]
        self.ref_cell = [4.6 * np.eye(3), 4.6 * np.eye(3)]
        self.ref_at = [1, 1]
        self.ref_e = [-1.90811235, -1.89718546]
        self.ref_f = [[[ 0.      ,  0.      ,  0.      ], \
                       [-0.      , -0.      , -0.      ]],\
                      [[-0.110216,  0.      ,  0.110216], \
                       [ 0.110216, -0.      , -0.110216]]]
        self.ref_v = [[[ 1.50816698,  0.        , -0.        ], \
                       [ 0.        ,  1.50816698,  0.        ], \
                       [-0.        ,  0.        ,  1.50816795]],\
                      [[ 1.45208913,  0.        ,  0.03036584], \
                       [ 0.        ,  1.67640928,  0.        ], \
                       [ 0.03036584,  0.        ,  1.45208913]]]
        self.ref_coord = np.array(self.ref_coord)
        self.ref_cell = np.array(self.ref_cell)
        self.ref_at = np.array(self.ref_at, dtype = int)
        self.ref_e = np.array(self.ref_e)
        self.ref_f = np.array(self.ref_f)
        self.ref_v = np.array(self.ref_v)

    def tearDown(self):
        shutil.rmtree(self.odir)

    def test_coll(self):

        with open (param_file, 'r') as fp :
            jdata = json.load (fp)
        jdata['out_dir'] = self.odir
        coll_vasp_md(jdata)
        
        sys = dpdata.LabeledSystem(self.odir + '/02.md/sys-004/deepmd//', fmt = 'deepmd/raw')
        self.assertEqual(sys.get_nframes(), 2)
        
        if sys.data['coords'][0][1][0] < sys.data['coords'][1][1][0]:
            idx = [1, 0]
        else :
            idx = [0, 1]
        ref_coord = self.ref_coord[idx]
        ref_cell = self.ref_cell[idx]
        ref_e = self.ref_e[idx]
        ref_f = self.ref_f[idx]
        ref_v = self.ref_v[idx]
        ref_at = self.ref_at
            
        for ff in range(2) :
            self.assertAlmostEqual(ref_e[ff], sys.data['energies'][ff])
        for ii in range(2) :
            self.assertEqual(ref_at[ff], sys.data['atom_types'][ff])
        for ff in range(2) :
            for ii in range(2) :
                for dd in range(3) :
                    self.assertAlmostEqual(ref_coord[ff][ii][dd], 
                                           sys.data['coords'][ff][ii][dd])
                    self.assertAlmostEqual(ref_f[ff][ii][dd], 
                                           sys.data['forces'][ff][ii][dd])
        for ff in range(2):
            for ii in range(3) :
                for jj in range(3) :
                    self.assertAlmostEqual(ref_v[ff][ii][jj], 
                                           sys.data['virials'][ff][ii][jj], places = 5)
                    self.assertAlmostEqual(ref_cell[ff][ii][jj], 
                                           sys.data['cells'][ff][ii][jj])

        

    



from   __future__ import unicode_literals, print_function
import logging
import os


ROOT_PATH=__path__[0]
NAME="dpgen"
SHORT_CMD="dpgen"
dlog = logging.getLogger(__name__)
dlog.setLevel(logging.INFO)
dlogf = logging.FileHandler(os.getcwd()+os.sep+SHORT_CMD+'.log')
dlogf_formatter=logging.Formatter('%(asctime)s - %(levelname)s : %(message)s')
#dlogf_formatter=logging.Formatter('%(asctime)s - %(name)s - [%(filename)s:%(funcName)s - %(lineno)d ] - %(levelname)s \n %(message)s')
dlogf.setFormatter(dlogf_formatter)
dlog.addHandler(dlogf)

__author__    = "Han Wang"
__copyright__ = "Copyright 2019"
__status__    = "Development"
try:
    from ._version import version as __version__
except ImportError:
    __version__ = 'unkown'
try:
    from ._date import date as __date__
except ImportError:
    __date__ = 'unkown'

def info():
    """
        Show basic information about """+NAME+""", its location and version.
    """

    print('DeepModeling\n------------')
    print('Version: ' + __version__)
    print('Date:    ' + __date__)
    print('Path:    ' + ROOT_PATH)
    print('')
    print('Dependency')
    print('------------')
    for modui in ['numpy', 'dpdata', 'pymatgen', 'monty', 'ase', 'paramiko', 'custodian' ]:
        try:
            mm = __import__(modui)
            print('%10s %10s   %s' % (modui, mm.__version__, mm.__path__[0]))
        except ImportError:
            print('%10s %10s Not Found' % (modui, ''))
    print()

    # reference
    print("""Reference
------------
Please cite:
Yuzhi Zhang, Haidi Wang, Weijie Chen, Jinzhe Zeng, Linfeng Zhang, Han Wang, and Weinan E,
DP-GEN: A concurrent learning platform for the generation of reliable deep learning
based potential energy models, Computer Physics Communications, 2020, 107206.
------------
""")



import sys,os

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..', '..')))
from dpgen.generator.run import *
from dpgen.generator.lib.gaussian import detect_multiplicity
from dpgen.generator.lib.ele_temp import NBandsEsti

param_file = 'param-mg-vasp.json'
param_file_v1 = 'param-mg-vasp-v1.json'
param_file_v1_et = 'param-mg-vasp-v1-et.json'
param_old_file = 'param-mg-vasp-old.json'
param_pwscf_file = 'param-pyridine-pwscf.json'
param_pwscf_old_file = 'param-pyridine-pwscf-old.json'
param_gaussian_file = 'param-pyridine-gaussian.json'
param_siesta_file = 'param-pyridine-siesta.json'
param_cp2k_file = 'param-pyridine-cp2k.json'
param_cp2k_file_v1 = 'param-pyridine-cp2k-choose-vdw.json'
machine_file = 'machine-local.json'
machine_file_v1 = 'machine-local-v1.json'
param_diy_file = 'param-mg-vasp-diy.json'
param_pwmat_file = 'param-pyridine-pwmat.json'

def my_file_cmp(test, f0, f1):
    with open(f0) as fp0 :
        with open(f1) as fp1:
            test.assertTrue(fp0.read() == fp1.read())

def setUpModule():
    os.chdir(os.path.abspath(os.path.dirname(__file__)))



#!/usr/bin/env python3

import os, re, argparse, filecmp, json, glob, sys
import subprocess as sp
import numpy as np
import dpgen.auto_test.lib.util as util
import dpgen.auto_test.lib.vasp as vasp
import dpgen.auto_test.lib.lammps as lammps

global_equi_name = '00.equi'
global_task_name = '05.surf'

def cmpt_vasp(jdata, conf_dir, static = False) :

    if 'relax_incar' in jdata.keys():
        vasp_str='vasp-relax_incar'
    else:
        kspacing = jdata['vasp_params']['kspacing']
        vasp_str='vasp-k%.2f' % (kspacing)

    equi_path = re.sub('confs', global_equi_name, conf_dir)
    equi_path = os.path.join(equi_path, vasp_str)
    equi_path = os.path.abspath(equi_path)
    equi_outcar = os.path.join(equi_path, 'OUTCAR')
    task_path = re.sub('confs', global_task_name, conf_dir)
    if static :
        if 'scf_incar' in jdata.keys():
            vasp_static_str='vasp-static-scf_incar'
        else:
            kspacing = jdata['vasp_params']['kspacing']
            vasp_static_str='vasp-static-k%.2f' % (kspacing)
        task_path = os.path.join(task_path, vasp_static_str)
    else :
        task_path = os.path.join(task_path, vasp_str)
    task_path = os.path.abspath(task_path)

    equi_natoms, equi_epa, equi_vpa = vasp.get_nev(equi_outcar)

    struct_path_widecard = os.path.join(task_path, 'struct-*-m*m')
    struct_path_list = glob.glob(struct_path_widecard)
    struct_path_list.sort()
    if len(struct_path_list) == 0:
        print("# cannot find results for conf %s" % (conf_dir))
    sys.stdout.write ("Miller_Indices: \tSurf_E(J/m^2) EpA(eV) equi_EpA(eV)\n")

    result = os.path.join(task_path,'result')
    with open(result,'w') as fp:
        fp.write('conf_dir:%s\n'% (conf_dir))
        fp.write("Miller_Indices: \tSurf_E(J/m^2) EpA(eV) equi_EpA(eV)\n")
        for ii in struct_path_list :
            structure_dir = os.path.basename(ii)
            outcar = os.path.join(ii, 'OUTCAR')
            natoms, epa, vpa = vasp.get_nev(outcar)
            if static :
                e0 = np.array(vasp.get_energies(outcar)) / natoms
                epa = e0[0]
            boxes = vasp.get_boxes(outcar)
            AA = np.linalg.norm(np.cross(boxes[0][0], boxes[0][1]))
            Cf = 1.60217657e-16 / (1e-20 * 2) * 0.001
            evac = (epa * natoms - equi_epa * natoms) / AA * Cf
            sys.stdout.write ("%s:\t %7.3f   %8.3f %8.3f\n" % (structure_dir, evac, epa, equi_epa))
            fp.write("%s:\t %7.3f   %8.3f %8.3f\n" % (structure_dir, evac, epa, equi_epa))
    fp.close()
    if 'upload_username' in jdata.keys():
        upload_username=jdata['upload_username']
        util.insert_data('surf','vasp',upload_username,result)

def cmpt_deepmd_lammps(jdata, conf_dir, task_name, static = False) :
    equi_path = re.sub('confs', global_equi_name, conf_dir)
    equi_path = os.path.join(equi_path, task_name.split('-')[0])
    equi_path = os.path.abspath(equi_path)
    equi_log = os.path.join(equi_path, 'log.lammps')
    task_path = re.sub('confs', global_task_name, conf_dir)
    task_path = os.path.join(task_path, task_name)
    task_path = os.path.abspath(task_path)

    equi_natoms, equi_epa, equi_vpa = lammps.get_nev(equi_log)

    struct_path_widecard = os.path.join(task_path, 'struct-*-m*m')
    struct_path_list = glob.glob(struct_path_widecard)
    struct_path_list.sort()
    if len(struct_path_list) == 0:
        print("# cannot find results for conf %s" % (conf_dir))
    sys.stdout.write ("Miller_Indices: \tSurf_E(J/m^2) EpA(eV) equi_EpA(eV)\n")
    result = os.path.join(task_path,'result')
    with open(result,'w') as fp:
        fp.write('conf_dir:%s\n'% (conf_dir))
        fp.write("Miller_Indices: \tSurf_E(J/m^2) EpA(eV) equi_EpA(eV)\n")
        for ii in struct_path_list :
            structure_dir = os.path.basename(ii)
            lmp_log = os.path.join(ii, 'log.lammps')
            natoms, epa, vpa = lammps.get_nev(lmp_log)
            AA = lammps.get_base_area(lmp_log)
            Cf = 1.60217657e-16 / (1e-20 * 2) * 0.001
            evac = (epa * natoms - equi_epa * natoms) / AA * Cf
            sys.stdout.write ("%s: \t%7.3f    %8.3f %8.3f\n" % (structure_dir, evac, epa, equi_epa))
            fp.write("%s:\t %7.3f   %8.3f %8.3f\n" % (structure_dir, evac, epa, equi_epa))
    fp.close()
    if 'upload_username' in jdata.keys() and task_name=='deepmd':
        upload_username=jdata['upload_username']
        util.insert_data('surf','deepmd',upload_username,result)

def _main() :
    parser = argparse.ArgumentParser(
        description="cmpt 05.surf")
    parser.add_argument('TASK', type=str,
                        help='the task of generation, vasp or lammps')
    parser.add_argument('PARAM', type=str,
                        help='json parameter file')
    parser.add_argument('CONF', type=str,
                        help='the path to conf')
    args = parser.parse_args()

    with open (args.PARAM, 'r') as fp :
        jdata = json.load (fp)

    print('# generate %s task with conf %s' % (args.TASK, args.CONF))
    if args.TASK == 'vasp':
        cmpt_vasp(jdata, args.CONF)
    elif args.TASK == 'vasp-static':
        cmpt_vasp(jdata, args.CONF, static = True)
    elif args.TASK == 'deepmd' :
        cmpt_deepmd_lammps(jdata, args.CONF, args.TASK)
    elif args.TASK == 'deepmd-static' :
        cmpt_deepmd_lammps(jdata, args.CONF, args.TASK, static = True)
    elif args.TASK == 'meam' :
        cmpt_deepmd_lammps(jdata, args.CONF, args.TASK)
    elif args.TASK == 'meam-static' :
        cmpt_deepmd_lammps(jdata, args.CONF, args.TASK, static = True)
    else :
        raise RuntimeError("unknow task ", args.TASK)

if __name__ == '__main__' :
    _main()



#!/usr/bin/env python3

import os,sys,json,glob,argparse,shutil
import numpy as np
import subprocess as sp
sys.path.append(os.path.join(os.path.dirname(os.path.realpath(__file__)), '..'))
from dpgen.tools.relabel import get_lmp_info

def ascii_hist(count) :
    np = (count-1) // 5 + 1
    ret = " |"
    for ii in range(np):
        ret += '='
    return ret    

def stat_sys(target_folder, 
             param_file = 'param.json', 
             verbose = True, 
             mute = False) :
    target_folder = os.path.abspath(target_folder)
    with open(os.path.join(target_folder, param_file)) as fp:
        jdata = json.load(fp)
    # goto input        
    cwd = os.getcwd()
    os.chdir(target_folder)
    sys = jdata['sys_configs']    
    numb_sys = len(sys)
    sys_tasks_count = [0 for ii in sys]
    sys_tasks_trait = [[] for ii in sys]
    sys_tasks_trait_count = [[] for ii in sys]
    # collect tasks from iter dirs
    iters = glob.glob('iter.[0-9]*[0-9]')
    iters.sort()
    # iters = iters[:2]
    for ii in iters :
        iter_tasks = glob.glob(os.path.join(ii, '02.fp', 'task.[0-9]*[0-9].[0-9]*[0-9]'))
        iter_tasks.sort()
        if verbose :
            print('# check iter ' + ii + ' with %6d tasks' % len(iter_tasks))
        for jj in iter_tasks :
            sys_idx = int(os.path.basename(jj).split('.')[-2])
            sys_tasks_count[sys_idx] += 1
            linked_file = os.path.realpath(os.path.join(jj, 'conf.dump'))
            linked_keys = linked_file.split('/')
            task_record = linked_keys[-5] + '.' + linked_keys[-3] + '.' + linked_keys[-1].split('.')[0]
            task_record_keys = task_record.split('.')
            ens, temp, pres = get_lmp_info(os.path.join(ii, '01.model_devi', linked_keys[-3], 'input.lammps'))
            trait = [ens, temp, pres]
            if not trait in sys_tasks_trait[sys_idx] :
                sys_tasks_trait[sys_idx].append(trait)
                sys_tasks_trait_count[sys_idx].append(0)
            t_idx = sys_tasks_trait[sys_idx].index(trait)
            sys_tasks_trait_count[sys_idx][t_idx] += 1
    sys_tasks_all = []
    for ii in range(numb_sys) :
        # print(sys[ii], sys_tasks_count[ii])
        tmp_all = []
        for jj in range(len(sys_tasks_trait[ii])) :
            tmp_all.append(sys_tasks_trait[ii][jj] + [sys_tasks_trait_count[ii][jj]])
        sys_tasks_all.append(tmp_all)
    for ii in sys_tasks_all:
        ii.sort()
    max_str_len = max([len(str(ii)) for ii in sys])
    sys_fmt = '%%%ds   %%6d' % (max_str_len+1)
    blank = max_str_len - 50
    str_blk = ""
    for ii in range(blank):
        str_blk += " "
    trait_fmt = str_blk + 'ens: %s   T: %10.2f   P: %12.2f   count:   %6d'
    for ii in range(numb_sys):
        if not mute:
            print(sys_fmt % (str(sys[ii]), sys_tasks_count[ii]))
        for jj in range(len(sys_tasks_all[ii])):
            hist_str = ascii_hist(sys_tasks_all[ii][jj][3])
            if not mute:
                print((trait_fmt + hist_str) % (sys_tasks_all[ii][jj][0],
                                                sys_tasks_all[ii][jj][1],
                                                sys_tasks_all[ii][jj][2],
                                                sys_tasks_all[ii][jj][3]))
    os.chdir(cwd)
    return sys, sys_tasks_count, sys_tasks_all

def run_report(args):
    stat_tasks(args.JOB_DIR, args.param, args.verbose)            





import numpy as np
from dpdata.periodic_table import Element



def _make_siesta_01_common(sys_data, ecut, ediff, mixingWeight, NumberPulay):
    tot_natoms = sum(sys_data['atom_numbs'])
    ntypes = len(sys_data['atom_names'])
    ret = ""
    ret += 'SystemName        system\n'
    ret += 'SystemLabel       system\n'
    ret += 'NumberOfAtoms     %d' % tot_natoms
    ret += '\nNumberOfSpecies   %d\n' % ntypes
    ret += '\n'
    ret += 'WriteForces       T\n'
    ret += 'WriteCoorStep     T\n'
    ret += 'WriteCoorXmol     T\n'
    ret += 'WriteMDXmol       T\n'
    ret += 'WriteMDHistory    T\n\n'

    ret += 'MeshCutoff            %s' % str(ecut)
    ret += ' Ry\n'
    ret += 'DM.MixingWeight       %f\n' % mixingWeight
    ret += 'DM.Tolerance          %e\n' % ediff
    ret += 'DM.UseSaveDM          true\n'
    ret += 'DM.NumberPulay         %d\n' % NumberPulay
    ret += 'MD.UseSaveXV           T\n\n'

    ret += 'XC.functional          GGA\n'
    ret += 'XC.authors             PBE\n'
    ret += 'DM.UseSaveDM           F\n'
    ret += 'WriteDM                F\n'
    ret += 'WriteDM.NetCDF         F\n'
    ret += 'WriteDMHS.NetCDF       F\n'
    return ret


def _make_siesta_02_species(sys_data, pps):
    atom_nums = sys_data['atom_numbs']
    atom_names = sys_data['atom_names']
    ntypes = len(atom_nums)
    assert (ntypes == len(atom_names))
    assert (ntypes == len(pps))
    ret = ''
    ret += '%block Chemical_Species_label\n'
    for i in range(0, len(atom_names)):
        ret += str(i + 1) + '\t' + str(Element(atom_names[i]).Z) + '\t' + atom_names[i] + '\n'
    ret += '%endblock Chemical_Species_label\n'
    return ret


# ## kpoints !!! can not understand
def _make_siesta_03_kpoint(sys_data, kspacing):
    cell = sys_data['cells'][0]
    cell = np.reshape(cell, [3, 3])
    ## np.linalg.inv()：矩阵求逆
    rcell = np.linalg.inv(cell)
    ## .T 矩阵转置
    rcell = rcell.T
    # np.ceil()是向上取整，与四舍五入无关 -5.6 --> -5
    # np.linalg.norm：进行范数运算，范数是对向量（或者矩阵）的度量，是一个标量（scalar）
    kpoints = [(np.ceil(2 * np.pi * np.linalg.norm(ii) / kspacing).astype(int))
               for ii in rcell]
    ret = ""
    ret += '%block kgrid_Monkhorst_Pack\n'
    ret += '%d' % kpoints[0]
    ret += '\t0\t0\t0.0\n'

    ret += '0\t'
    ret += '%d' % kpoints[1]
    ret += '\t0\t0.0\n'

    ret += '0\t0\t'
    ret += '%d' % kpoints[2]
    ret += '\t0.0\n'

    ret += '%endblock kgrid_Monkhorst_Pack\n'
    return ret


### coordinate
def _make_siesta_04_ucVectorCoord(sys_data):
    cell = sys_data['cells'][0]
    cell = np.reshape(cell, [3, 3])
    coordinates = sys_data['coords'][0]
    atom_names = (sys_data['atom_names'])
    atom_numbs = (sys_data['atom_numbs'])
    ntypes = len(atom_names)
    ret = ""
    ret += "LatticeConstant try_input_output.00 Ang\n"
    ret += "%block LatticeVectors\n"
    for ii in range(3):
        for jj in range(3):
            ret += "%f " % cell[ii][jj]
        ret += "\n"
    ret += "%endblock LatticeVectors\n"

    ret += "\n"
    ret += "AtomicCoordinatesFormat Ang\n"
    ret += "%block AtomicCoordinatesAndAtomicSpecies\n"
    cc = 0
    for ii in range(ntypes):
        for jj in range(atom_numbs[ii]):
            ret += "%f %f %f %d %s\n" % (coordinates[cc][0],
                                         coordinates[cc][1],
                                         coordinates[cc][2],
                                         ii + 1,
                                         atom_names[ii])
            cc += 1
    ret += "%endblock AtomicCoordinatesAndAtomicSpecies"
    return ret

def make_siesta_input(sys_data, fp_pp_files, fp_params):
    ecut = fp_params['ecut']
    ediff = fp_params['ediff']
    mixingWeight = fp_params['mixingWeight']
    NumberPulay = fp_params['NumberPulay']
    kspacing = fp_params['kspacing']
    ret = ""
    ret += _make_siesta_01_common(sys_data, ecut, ediff, mixingWeight, NumberPulay)
    ret += "\n"
    ret += _make_siesta_02_species(sys_data, fp_pp_files)
    ret += "\n"
    ret += _make_siesta_03_kpoint(sys_data, kspacing)
    ret += "\n"
    ret += _make_siesta_04_ucVectorCoord(sys_data)
    ret += "\n"
    return ret




import os,sys
import dpdata
import numpy as np
import unittest
import importlib

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))
__package__ = 'generator'
from .context import NBandsEsti

class TestNBandsEsti(unittest.TestCase):    
    def test_predict(self):
        self.nbe = NBandsEsti(['out_data_nbands_esti/md.010000K',
                               'out_data_nbands_esti/md.020000K', 
                               'out_data_nbands_esti/md.040000K', 
                               'out_data_nbands_esti/md.080000K', 
                               'out_data_nbands_esti/md.160000K', 
        ])
        self.assertEqual(self.nbe.predict('out_data_nbands_esti/md.010000K'), 72)
        self.assertEqual(self.nbe.predict('out_data_nbands_esti/md.020000K'), 83)
        self.assertEqual(self.nbe.predict('out_data_nbands_esti/md.040000K'), 112)
        self.assertEqual(self.nbe.predict('out_data_nbands_esti/md.080000K'), 195)
        self.assertEqual(self.nbe.predict('out_data_nbands_esti/md.160000K'), 429)
        self.assertEqual(self.nbe.predict('out_data_nbands_esti/md.240000K'), 732)

    def test_save_load(self):
        self.nbe2 = NBandsEsti(['out_data_nbands_esti/md.010000K',
                               'out_data_nbands_esti/md.020000K', 
                               'out_data_nbands_esti/md.040000K', 
                               'out_data_nbands_esti/md.080000K', 
                               'out_data_nbands_esti/md.160000K', 
        ])
        self.nbe2.save('tmp.log')
        self.nbe = NBandsEsti('tmp.log')
        self.assertEqual(self.nbe.predict('out_data_nbands_esti/md.010000K'), 72)
        self.assertEqual(self.nbe.predict('out_data_nbands_esti/md.020000K'), 83)
        self.assertEqual(self.nbe.predict('out_data_nbands_esti/md.040000K'), 112)
        self.assertEqual(self.nbe.predict('out_data_nbands_esti/md.080000K'), 195)
        self.assertEqual(self.nbe.predict('out_data_nbands_esti/md.160000K'), 429)
        self.assertEqual(self.nbe.predict('out_data_nbands_esti/md.240000K'), 732)
        os.remove('tmp.log')

    def test_get_default_nbands(self):
        res = NBandsEsti._get_res('out_data_nbands_esti/md.020000K/')
        nb = NBandsEsti._get_default_nbands(res)
        self.assertEqual(nb, 66)

    def test_get_default_nbands(self):
        res = NBandsEsti._get_res('out_data_nbands_esti/mgal/')
        nb = NBandsEsti._get_default_nbands(res)
        self.assertEqual(nb, 124)

    def test_potcar_nvalence (self) :
        res = NBandsEsti._get_potcar_nvalence('out_data_nbands_esti/POTCAR.dbl')
        self.assertEqual(res, [10., 3.])

    def test_incar_ele_temp (self) :
        res = NBandsEsti._get_incar_ele_temp('out_data_nbands_esti/md.000300K/INCAR')
        self.assertAlmostEqual(res, 0.025851991011651636)

    def test_incar_nbands (self) :
        res = NBandsEsti._get_incar_nbands('out_data_nbands_esti/md.020000K/INCAR')
        self.assertEqual(res, 81)

    def test_get_res(self):
        res = NBandsEsti._get_res('out_data_nbands_esti/md.020000K/')
        ref = {
            'natoms': [32],
            'vol': 138.55418502346618,
            'nvalence': [3.],
            'ele_temp': 20000.0,
            'nbands': 81
        }
        self.assertEqual(res['natoms'], ref['natoms'])
        self.assertAlmostEqual(res['vol'], ref['vol'])
        self.assertAlmostEqual(res['nvalence'][0], ref['nvalence'][0])
        self.assertEqual(len(res['nvalence']), len(ref['nvalence']))
        self.assertAlmostEqual(res['ele_temp'], ref['ele_temp'], places = 1)
        self.assertEqual(res['nbands'], ref['nbands'])    



import subprocess
import os
def stat_time(target_folder, 
            param_file = 'param.json', 
            verbose = True, 
            mute = False):
    train_dirs = subprocess.run([f"ls -d -1 {target_folder}/iter.??????/00.train/", ],
        shell=True,stdout=subprocess.PIPE).stdout.decode().strip().split('\n')
    for dir in train_dirs:
        abs_dir = os.path.abspath(dir)
        stage = os.path.basename(os.path.dirname(dir))
        train_time_logs = subprocess.run([f"grep -H --text 'wall time' {dir}/???/train.log", ],
            shell=True,stdout=subprocess.PIPE).stdout.decode().strip().split('\n')
        upload_task_dir_num = subprocess.run([f"ls -1 -d {dir}/??? |wc -l", ], 
            shell=True, stdout=subprocess.PIPE).stdout.decode().strip('\n')
        total_core_sec = float(0)
        
        # assume training on single GPU
        paral_cores = 1
        finished_task_file_num = len(train_time_logs)
        # gpu_type_set = set([])
        for log in train_time_logs:
            # log example : 
            #   .//iter.000000/00.train//003/train.log:# DEEPMD: wall time: 7960.265 s
            # print(log.split(':'))
            file_path, text1, text2, wall_time = log.split(':') # pylint: disable=unused-variable
            abs_file_path = os.path.abspath(file_path)
            # stage=='00.train'
            
            wall_time_sec = float(wall_time.strip('s').strip(' '))
            total_core_sec += wall_time_sec * paral_cores

            # r'd\nja\1lgje' leading 'r' means dont treat '\' as  Escape character
            # gpu_type = subprocess.run([fr"grep -e 'physical GPU' {abs_file_path} |sed -n -E -e 's|^.*name: (.*), pci.*|\1|p'", ],
            #     shell=True,stdout=subprocess.PIPE).stdout.decode().strip().split('\n').pop()
            # gpu_type_set.add(gpu_type)
        
        total_core_hour = total_core_sec * paral_cores / 3600 
        print(f"{stage}:{abs_dir}"
            f"paral_cores:{paral_cores}"
            f":upload_task_dir_num:{upload_task_dir_num}"
            f":finished_task_file_num:{finished_task_file_num}"
            f":total_core_hour:{total_core_hour:.3f}")
    
    model_devi_dirs = subprocess.run([f"ls -d -1 {target_folder}/iter.??????/01.model_devi/", ],
        shell=True,stdout=subprocess.PIPE).stdout.decode().strip().split('\n')
    # print(model_devi_dirs)
    for dir in model_devi_dirs:
        abs_dir = os.path.abspath(dir)
        stage = os.path.basename(os.path.dirname(dir))
        # print(dir)
        model_devi_time_logs = subprocess.run([f"grep -H --text 'wall time' {dir}/task.*/log.lammps", ],
            shell=True,stdout=subprocess.PIPE).stdout.decode().strip().split('\n')
        upload_task_dir_num = subprocess.run([f"ls -1 -d {dir}/task.* |wc -l", ], 
            shell=True, stdout=subprocess.PIPE).stdout.decode().strip('\n')
        total_core_sec = float(0)
        finished_task_file_num = len(model_devi_time_logs)
        # assume model_devi lammps job running on GPUs , set paral_cores==1
        paral_cores = 1 
        for log in model_devi_time_logs:
            # log example:
            #   .//iter.000002/01.model_devi//task.018.000075/log.lammps:Total wall time: 0:00:39
            # print(log)
            file_path, text1, hour, min, sec = log.split(':') # pylint: disable=unused-variable
            abs_file_path =  os.path.abspath(file_path)
            wall_time_sec = 3600*int(hour) + 60*int(min) + 1*int(sec)
            total_core_sec += wall_time_sec * paral_cores
        total_core_hour = total_core_sec / 3600

        print(f"{stage}:{abs_dir}"
            f":paral_cores:{paral_cores}"
            f":upload_task_dir_num:{upload_task_dir_num}"
            f":finished_task_file_num:{finished_task_file_num}"
            f":total_core_hour:{total_core_hour:.3f}")

    fp_dirs = subprocess.run([f"ls -d -1 {target_folder}/iter.??????/02.fp/", ],
        shell=True,stdout=subprocess.PIPE).stdout.decode().strip().split('\n')
    for dir in fp_dirs:
        abs_dir = os.path.abspath(dir)
        stage = os.path.basename(os.path.dirname(dir))
        fp_time_logs = subprocess.run([f"grep -H --text 'CPU time' {dir}/task.*/OUTCAR", ],
            shell=True,stdout=subprocess.PIPE).stdout.decode().strip().split('\n')
        upload_task_dir_num = subprocess.run([f"ls -1 -d {dir}/task.* |wc -l", ], 
            shell=True, stdout=subprocess.PIPE).stdout.decode().strip('\n')
        total_core_sec = float(0)
        finished_task_file_num = len(fp_time_logs)
        for log in fp_time_logs:
            # log example:
            #   .//iter.000002/02.fp//task.018.000048/OUTCAR:                  Total CPU time used (sec):      288.395
            file_path, text1, sec = log.split(':')
            abs_file_path = os.path.abspath(file_path)
            wall_time_sec = float(sec)
            paral_cores = subprocess.run([fr"head -n 1000 {abs_file_path} | grep 'running on' | sed -n -E -e 's|running on\s+([0-9]+)+\s.*|\1|p' ", ],
                shell=True,stdout=subprocess.PIPE).stdout.decode().strip()
            total_core_sec += wall_time_sec * int(paral_cores)
        total_core_hour = total_core_sec /3600

        print(f"{stage}:{abs_dir}"
            f":paral_cores:{paral_cores}"
            f":upload_task_dir_num:{upload_task_dir_num}"
            f":finished_task_file_num:{finished_task_file_num}"
            f":total_core_hour:{total_core_hour:.3f}")

if __name__=='__main__':
    stat_time(target_folder="./")